@@ -1,7 +1,6 @@
 #include <ATen/CPUGeneratorImpl.h>
 #include <ATen/Utils.h>
 #include <ATen/core/MT19937RNGEngine.h>
-#include <c10/util/C++17.h>
 #include <c10/util/MathConstants.h>
 #include <algorithm>
 
@@ -22,6 +21,7 @@ struct CPUGeneratorImplStateLegacy {
   int left;  /* = 1; */
   int seeded; /* = 0; */
   uint64_t next;
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
   uint64_t state[at::MERSENNE_STATE_N]; /* the array for the state vector  */
 
   /********************************/
@@ -38,7 +38,7 @@ struct CPUGeneratorImplStateLegacy {
  * new data introduced in at::CPUGeneratorImpl and the legacy state. It is used
  * as a helper for torch.get_rng_state() and torch.set_rng_state()
  * functions.
- */ 
+ */
 struct CPUGeneratorImplState {
   CPUGeneratorImplStateLegacy legacy_pod;
   float next_float_normal_sample;
@@ -81,8 +81,8 @@ inline uint64_t make64BitsFrom32Bits(uint32_t hi, uint32_t lo) {
 CPUGeneratorImpl::CPUGeneratorImpl(uint64_t seed_in)
   : c10::GeneratorImpl{Device(DeviceType::CPU), DispatchKeySet(c10::DispatchKey::CPU)},
     engine_{seed_in},
-    next_float_normal_sample_{c10::optional<float>()},
-    next_double_normal_sample_{c10::optional<double>()} { }
+    next_float_normal_sample_{std::optional<float>()},
+    next_double_normal_sample_{std::optional<double>()} { }
 
 /**
  * Manually seeds the engine with the seed input
@@ -94,6 +94,21 @@ void CPUGeneratorImpl::set_current_seed(uint64_t seed) {
   engine_ = mt19937(seed);
 }
 
+/**
+ * Sets the offset of RNG state.
+ * See Note [Acquire lock when using random generators]
+ */
+void CPUGeneratorImpl::set_offset(uint64_t offset [[maybe_unused]]) {
+  TORCH_CHECK(false, "CPU Generator does not use offset");
+}
+
+/**
+ * Gets the current offset of CPUGeneratorImpl.
+ */
+uint64_t CPUGeneratorImpl::get_offset() const {
+  TORCH_CHECK(false, "CPU Generator does not use offset");
+}
+
 /**
  * Gets the current seed of CPUGeneratorImpl.
  */
@@ -119,15 +134,15 @@ uint64_t CPUGeneratorImpl::seed() {
  * must be a strided CPU byte tensor and of the same size as either
  * CPUGeneratorImplStateLegacy (for legacy CPU generator state) or
  * CPUGeneratorImplState (for new state).
- * 
+ *
  * FIXME: Remove support of the legacy state in the future?
  */
 void CPUGeneratorImpl::set_state(const c10::TensorImpl& new_state) {
   using detail::CPUGeneratorImplState;
   using detail::CPUGeneratorImplStateLegacy;
 
-  static_assert(std::is_pod<CPUGeneratorImplStateLegacy>::value, "CPUGeneratorImplStateLegacy is not a PODType");
-  static_assert(std::is_pod<CPUGeneratorImplState>::value, "CPUGeneratorImplState is not a PODType");
+  static_assert(std::is_standard_layout_v<CPUGeneratorImplStateLegacy>, "CPUGeneratorImplStateLegacy is not a PODType");
+  static_assert(std::is_standard_layout_v<CPUGeneratorImplState>, "CPUGeneratorImplState is not a PODType");
 
   static const size_t size_legacy = sizeof(CPUGeneratorImplStateLegacy);
   static const size_t size_current = sizeof(CPUGeneratorImplState);
@@ -136,16 +151,16 @@ void CPUGeneratorImpl::set_state(const c10::TensorImpl& new_state) {
   detail::check_rng_state(new_state);
 
   at::mt19937 engine;
-  auto float_normal_sample = c10::optional<float>();
-  auto double_normal_sample = c10::optional<double>();
+  auto float_normal_sample = std::optional<float>();
+  auto double_normal_sample = std::optional<double>();
 
   // Construct the state of at::CPUGeneratorImpl based on input byte tensor size.
-  CPUGeneratorImplStateLegacy* legacy_pod;
+  CPUGeneratorImplStateLegacy* legacy_pod{nullptr};
   auto new_state_size = new_state.numel();
   if (new_state_size == size_legacy) {
     legacy_pod = (CPUGeneratorImplStateLegacy*)new_state.data();
     // Note that in CPUGeneratorImplStateLegacy, we didn't have float version
-    // of normal sample and hence we leave the c10::optional<float> as is
+    // of normal sample and hence we leave the std::optional<float> as is
 
     // Update next_double_normal_sample.
     // Note that CPUGeneratorImplStateLegacy stores two uniform values (normal_x, normal_y)
@@ -156,14 +171,14 @@ void CPUGeneratorImpl::set_state(const c10::TensorImpl& new_state) {
       auto r = legacy_pod->normal_rho;
       auto theta = 2.0 * c10::pi<double> * legacy_pod->normal_x;
       // we return the sin version of the normal sample when in caching mode
-      double_normal_sample = c10::optional<double>(r * ::sin(theta));
+      double_normal_sample = std::optional<double>(r * ::sin(theta));
     }
   } else if (new_state_size == size_current) {
     auto rng_state = (CPUGeneratorImplState*)new_state.data();
     legacy_pod = &rng_state->legacy_pod;
     // update next_float_normal_sample
     if (rng_state->is_next_float_normal_sample_valid) {
-      float_normal_sample = c10::optional<float>(rng_state->next_float_normal_sample);
+      float_normal_sample = std::optional<float>(rng_state->next_float_normal_sample);
     }
 
     // Update next_double_normal_sample.
@@ -171,10 +186,10 @@ void CPUGeneratorImpl::set_state(const c10::TensorImpl& new_state) {
     // and if it's valid in normal_is_valid. The redundant normal_x and normal_rho
     // are squashed to 0.0.
     if (legacy_pod->normal_is_valid) {
-      double_normal_sample = c10::optional<double>(legacy_pod->normal_y);
+      double_normal_sample = std::optional<double>(legacy_pod->normal_y);
     }
   } else {
-    AT_ERROR("Expected either a CPUGeneratorImplStateLegacy of size ", size_legacy,
+    TORCH_CHECK(false, "Expected either a CPUGeneratorImplStateLegacy of size ", size_legacy,
              " or a CPUGeneratorImplState of size ", size_current,
              " but found the input RNG state size to be ", new_state_size);
   }
@@ -183,7 +198,7 @@ void CPUGeneratorImpl::set_state(const c10::TensorImpl& new_state) {
   // Note that CPUGeneratorImplStateLegacy stored a state array of 64 bit uints, whereas in our
   // redefined mt19937, we have changed to a state array of 32 bit uints. Hence, we are
   // doing a std::copy.
-  at::mt19937_data_pod rng_data;
+  at::mt19937_data_pod rng_data{};
   std::copy(std::begin(legacy_pod->state), std::end(legacy_pod->state), rng_data.state_.begin());
   rng_data.seed_ = legacy_pod->the_initial_seed;
   rng_data.left_ = legacy_pod->left;
@@ -204,9 +219,9 @@ c10::intrusive_ptr<c10::TensorImpl> CPUGeneratorImpl::get_state() const {
   using detail::CPUGeneratorImplState;
 
   static const size_t size = sizeof(CPUGeneratorImplState);
-  static_assert(std::is_pod<CPUGeneratorImplState>::value, "CPUGeneratorImplState is not a PODType");
+  static_assert(std::is_standard_layout_v<CPUGeneratorImplState>, "CPUGeneratorImplState is not a PODType");
 
-  auto state_tensor = at::detail::empty_cpu({(int64_t)size}, ScalarType::Byte, c10::nullopt, c10::nullopt, c10::nullopt, c10::nullopt);
+  auto state_tensor = at::detail::empty_cpu({(int64_t)size}, ScalarType::Byte, std::nullopt, std::nullopt, std::nullopt, std::nullopt);
   auto rng_state = state_tensor.data_ptr();
 
   // accumulate generator data to be copied into byte tensor
@@ -267,14 +282,14 @@ uint64_t CPUGeneratorImpl::random64() {
 /**
  * Get the cached normal random in float
  */
-c10::optional<float> CPUGeneratorImpl::next_float_normal_sample() {
+std::optional<float> CPUGeneratorImpl::next_float_normal_sample() {
   return next_float_normal_sample_;
 }
 
 /**
  * Get the cached normal random in double
  */
-c10::optional<double> CPUGeneratorImpl::next_double_normal_sample() {
+std::optional<double> CPUGeneratorImpl::next_double_normal_sample() {
   return next_double_normal_sample_;
 }
 
@@ -283,7 +298,7 @@ c10::optional<double> CPUGeneratorImpl::next_double_normal_sample() {
  *
  * See Note [Acquire lock when using random generators]
  */
-void CPUGeneratorImpl::set_next_float_normal_sample(c10::optional<float> randn) {
+void CPUGeneratorImpl::set_next_float_normal_sample(std::optional<float> randn) {
   next_float_normal_sample_ = randn;
 }
 
@@ -292,7 +307,7 @@ void CPUGeneratorImpl::set_next_float_normal_sample(c10::optional<float> randn)
  *
  * See Note [Acquire lock when using random generators]
  */
-void CPUGeneratorImpl::set_next_double_normal_sample(c10::optional<double> randn) {
+void CPUGeneratorImpl::set_next_double_normal_sample(std::optional<double> randn) {
   next_double_normal_sample_ = randn;
 }
 
