@@ -6,7 +6,7 @@
 #endif
 
 #if AT_MKLDNN_ENABLED()
-#include <mkldnn.hpp>
+#include <dnnl.hpp>
 #include <ideep.hpp>
 #endif
 
@@ -24,9 +24,9 @@ std::string get_mkl_version() {
     {
       // Magic buffer number is from MKL documentation
       // https://software.intel.com/en-us/mkl-developer-reference-c-mkl-get-version-string
-      char buf[198];
-      mkl_get_version_string(buf, 198);
-      version = buf;
+      version.resize(198,'\0');
+      mkl_get_version_string(version.data(), 198);
+      version.resize(strlen(version.c_str()));
     }
   #else
     version = "MKL not found";
@@ -42,7 +42,7 @@ std::string get_mkldnn_version() {
     // Apparently no way to get ideep version?
     // https://github.com/intel/ideep/issues/29
     {
-      const mkldnn_version_t* ver = mkldnn_version();
+      const dnnl_version_t* ver = dnnl_version();
       ss << "Intel(R) MKL-DNN v" << ver->major << "." << ver->minor << "." << ver->patch
          << " (Git Hash " << ver->hash << ")";
     }
@@ -90,34 +90,45 @@ std::string get_openmp_version() {
   return ss.str();
 }
 
-std::string used_cpu_capability() {
+std::string get_cpu_capability() {
   // It is possible that we override the cpu_capability with
   // environment variable
-  std::ostringstream ss;
-  ss << "CPU capability usage: ";
   auto capability = native::get_cpu_capability();
   switch (capability) {
-#ifdef HAVE_VSX_CPU_DEFINITION
+#if defined(HAVE_VSX_CPU_DEFINITION)
     case native::CPUCapability::DEFAULT:
-      ss << "DEFAULT";
-      break;
+      return "DEFAULT";
     case native::CPUCapability::VSX:
-      ss << "VSX";
-      break;
+      return "VSX";
+#elif defined(HAVE_ZVECTOR_CPU_DEFINITION)
+    case native::CPUCapability::DEFAULT:
+      return "DEFAULT";
+    case native::CPUCapability::ZVECTOR:
+      return "Z VECTOR";
+#elif defined(HAVE_SVE_CPU_DEFINITION)
+    case native::CPUCapability::DEFAULT:
+      return "DEFAULT";
+    case native::CPUCapability::SVE256:
+      return "SVE256";
 #else
     case native::CPUCapability::DEFAULT:
-      ss << "NO AVX";
-      break;
-    case native::CPUCapability::AVX:
-      ss << "AVX";
-      break;
+      return "NO AVX";
     case native::CPUCapability::AVX2:
-      ss << "AVX2";
-      break;
-#endif      
+      return "AVX2";
+    case native::CPUCapability::AVX512:
+      return "AVX512";
+#endif
     default:
       break;
   }
+  return "";
+}
+
+static std::string used_cpu_capability() {
+  // It is possible that we override the cpu_capability with
+  // environment variable
+  std::ostringstream ss;
+  ss << "CPU capability usage: " << get_cpu_capability();
   return ss.str();
 }
 
@@ -164,7 +175,7 @@ std::string show_config() {
   ss << "  - " << get_openmp_version() << "\n";
 #endif
 
-#ifdef USE_LAPACK
+#if AT_BUILD_WITH_LAPACK()
   // TODO: Actually record which one we actually picked
   ss << "  - LAPACK is enabled (usually provided by MKL)\n";
 #endif
@@ -184,6 +195,14 @@ std::string show_config() {
     ss << detail::getCUDAHooks().showConfig();
   }
 
+  if (hasMAIA()) {
+    ss << detail::getMAIAHooks().showConfig();
+  }
+
+  if (hasXPU()) {
+    ss << detail::getXPUHooks().showConfig();
+  }
+
   ss << "  - Build settings: ";
   for (const auto& pair : caffe2::GetBuildOptions()) {
     if (!pair.second.empty()) {
@@ -194,6 +213,7 @@ std::string show_config() {
 
   // TODO: do HIP
   // TODO: do XLA
+  // TODO: do MPS
 
   return ss.str();
 }
