@@ -1,7 +1,9 @@
-import operator_benchmark as op_bench
-import torch
 import numpy
 
+import operator_benchmark as op_bench
+
+import torch
+
 
 """Microbenchmarks for gather operator."""
 
@@ -13,36 +15,35 @@ gather_configs_short = op_bench.config_list(
         [512, 512, 1],
     ],
     cross_product_configs={
-        'device': ['cpu', 'cuda'],
+        "device": ["cpu", "cuda"],
     },
-    tags=["short"]
+    tags=["short"],
 )
 
 
 gather_configs_long = op_bench.cross_product_configs(
-    M=[128, 1024],
-    N=[128, 1024],
-    dim=[0, 1],
-    device=['cpu', 'cuda'],
-    tags=["long"]
+    M=[128, 1024], N=[128, 1024], dim=[0, 1], device=["cpu", "cuda"], tags=["long"]
 )
 
 
 class GatherBenchmark(op_bench.TorchBenchmarkBase):
     def init(self, M, N, dim, device):
-        self.input_one = torch.rand(M, N, device=device)
-        self.dim = dim
         min_val = M if dim == 0 else N
         numpy.random.seed((1 << 32) - 1)
-        self.index = torch.tensor(numpy.random.randint(0, min_val, (M, N)), device=device)
+        self.inputs = {
+            "input_one": torch.rand(M, N, device=device),
+            "dim": dim,
+            "index": torch.tensor(
+                numpy.random.randint(0, min_val, (M, N)), device=device
+            ),
+        }
         self.set_module_name("gather")
 
-    def forward(self):
-        return torch.gather(self.input_one, self.dim, self.index)
+    def forward(self, input_one, dim: int, index):
+        return torch.gather(input_one, dim, index)
 
 
-op_bench.generate_pt_test(gather_configs_short + gather_configs_long,
-                          GatherBenchmark)
+op_bench.generate_pt_test(gather_configs_short + gather_configs_long, GatherBenchmark)
 
 
 if __name__ == "__main__":
