@@ -1,8 +1,11 @@
-import torch
 import argparse
-from common import SubTensor, WithTorchFunction, SubWithTorchFunction  # noqa: F401
 
-Tensor = torch.Tensor
+from common import SubTensor, SubWithTorchFunction, WithTorchFunction  # noqa: F401
+
+import torch
+
+
+Tensor = torch.tensor
 
 NUM_REPEATS = 1000000
 
@@ -21,8 +24,8 @@ if __name__ == "__main__":
     TensorClass = globals()[args.tensor_class]
     NUM_REPEATS = args.nreps
 
-    t1 = TensorClass(1)
-    t2 = TensorClass(2)
+    t1 = TensorClass([1.0])
+    t2 = TensorClass([2.0])
 
     for _ in range(NUM_REPEATS):
         torch.add(t1, t2)
