@@ -1,19 +1,24 @@
-r'''
+r"""
 **This file is EXPERIMENTAL and is mostly used for testing purposes! Do not
 rely on it for anything!**
-'''
+"""
+import operator
+import sys
+
+import torch
 from torch.fx import Graph, GraphModule
 from torch.fx.graph import map_arg
 from torch.fx.proxy import Proxy
-import sys
-import torch
 from torch.nn.utils import fuse_conv_bn_weights
-import operator
+
 
 # can be a
 #  module type, a builtin function, or a string to match target
 
-def _minmax_scale_zeropoint(min_val, max_val, qmin=-127, qmax=128, eps=torch.finfo(torch.float32).eps):
+
+def _minmax_scale_zeropoint(
+    min_val, max_val, qmin=-127, qmax=128, eps=torch.finfo(torch.float32).eps
+):
     min_val = min(0.0, min_val)
     max_val = max(0.0, max_val)
     if max_val == min_val:
@@ -27,9 +32,10 @@ def _minmax_scale_zeropoint(min_val, max_val, qmin=-127, qmax=128, eps=torch.fin
         zero_point = int(zero_point)
         return scale, zero_point
 
+
 class MinMaxObserver:
     def __init__(self, quantizer, node):
-        self.min, self.max = float('inf'), float('-inf')
+        self.min, self.max = float("inf"), float("-inf")
         self.all_tensors = True
 
     def observe(self, node, env):
@@ -43,6 +49,7 @@ class MinMaxObserver:
     def scale_zeropoint(self):
         return _minmax_scale_zeropoint(self.min, self.max, qmin=0, qmax=255)
 
+
 class NoObserver:
     def __init__(self, quantizer, node):
         pass
@@ -50,11 +57,15 @@ class NoObserver:
     def observe(self, node, env):
         pass
 
-DEFAULT_QUANTIZATION_PATTERNS = {}
+
+_DEFAULT_QUANTIZATION_PATTERNS = {}
+
+
 def register_pattern(pattern):
     def insert(fn):
-        DEFAULT_QUANTIZATION_PATTERNS[pattern] = fn
+        _DEFAULT_QUANTIZATION_PATTERNS[pattern] = fn
         return fn
+
     return insert
 
 
@@ -65,12 +76,19 @@ class Add(MinMaxObserver):
             return NotImplemented
         scale, zeropoint = self.scale_zeropoint()
         return quantizer.quantized_graph.create_node(
-            'call_function', torch.ops.quantized.add, load_arg(node.args), {'scale': scale, 'zero_point': zeropoint})
+            "call_function",
+            torch.ops.quantized.add,
+            load_arg(node.args),
+            {"scale": scale, "zero_point": zeropoint},
+        )
 
 
 class Relu(NoObserver):
     def quantize(self, quantizer, node, load_arg):
-        return torch.relu(load_arg(node.args[0]))  # torch.relu works directly on quantized tensors?
+        return torch.relu(
+            load_arg(node.args[0])
+        )  # torch.relu works directly on quantized tensors?
+
 
 # these ops have quantized equivalents that do not need any extra information
 @register_pattern(torch.nn.ReLU)
@@ -81,15 +99,24 @@ class CopyNode(NoObserver):
     def quantize(self, quantizer, node, load_arg):
         return quantizer.quantized_graph.node_copy(node, load_arg)
 
+
 class IdentityModule(torch.nn.Module):
     def forward(self, x):
         return x
 
+
 # handle conv, maybe followed by bn, maybe followed by relu
 @register_pattern(torch.nn.modules.conv.Conv2d)
 @register_pattern((torch.nn.ReLU, torch.nn.modules.conv.Conv2d))
-@register_pattern((torch.nn.modules.batchnorm.BatchNorm2d, torch.nn.modules.conv.Conv2d))
-@register_pattern((torch.nn.ReLU, (torch.nn.modules.batchnorm.BatchNorm2d, torch.nn.modules.conv.Conv2d)))
+@register_pattern(
+    (torch.nn.modules.batchnorm.BatchNorm2d, torch.nn.modules.conv.Conv2d)
+)
+@register_pattern(
+    (
+        torch.nn.ReLU,
+        (torch.nn.modules.batchnorm.BatchNorm2d, torch.nn.modules.conv.Conv2d),
+    )
+)
 class ConvNormRelu(MinMaxObserver):
     def __init__(self, quantizer, node):
         super().__init__(quantizer, node)
@@ -111,21 +138,41 @@ class ConvNormRelu(MinMaxObserver):
 
         if self.bn_node is not None:
             weight, bias = fuse_conv_bn_weights(
-                weight, bias, self.bn.running_mean, self.bn.running_var,
-                self.bn.eps, self.bn.weight, self.bn.bias)
+                weight,
+                bias,
+                self.bn.running_mean,
+                self.bn.running_var,
+                self.bn.eps,
+                self.bn.weight,
+                self.bn.bias,
+            )
 
         min_val, max_val = float(weight.min()), float(weight.max())
 
         act_scale, act_zp = self.scale_zeropoint()
 
         weight_scale, weight_zp = _minmax_scale_zeropoint(min_val, max_val)
-        qweight = torch.quantize_per_tensor(weight, weight_scale, weight_zp, torch.qint8)
+        qweight = torch.quantize_per_tensor(
+            weight, weight_scale, weight_zp, torch.qint8
+        )
 
-        ctor = torch.nn.intrinsic.quantized.ConvReLU2d if self.relu_node is not None else torch.nn.quantized.Conv2d
+        ctor = (
+            torch.ao.nn.intrinsic.quantized.ConvReLU2d
+            if self.relu_node is not None
+            else torch.ao.nn.quantized.Conv2d
+        )
 
-        qconv = ctor(mod.in_channels, mod.out_channels, mod.kernel_size,
-                     mod.stride, mod.padding, mod.dilation, mod.groups,
-                     mod.bias is not None, mod.padding_mode)
+        qconv = ctor(
+            mod.in_channels,
+            mod.out_channels,
+            mod.kernel_size,
+            mod.stride,
+            mod.padding,
+            mod.dilation,
+            mod.groups,
+            mod.bias is not None,
+            mod.padding_mode,
+        )
 
         qconv.set_weight_bias(qweight, bias)
         qconv.scale = float(act_scale)
@@ -133,29 +180,36 @@ class ConvNormRelu(MinMaxObserver):
         parent_name, name = _parent_name(self.conv_node.target)
         setattr(quantizer.modules[parent_name], name, qconv)
         if self.bn_node is not None:
-            parent_bn, bn_name = _parent_name(self.bn_node.target)
+            _, bn_name = _parent_name(self.bn_node.target)
             # we can't just delete this because submodules's forwards (which are not longer use)
             # try to call it, so replace with something that does nothing.
             setattr(quantizer.modules[parent_name], bn_name, IdentityModule())
 
-        return quantizer.quantized_graph.create_node('call_module', self.conv_node.target, (load_arg(self.conv_node.args[0]),), {})
+        return quantizer.quantized_graph.create_node(
+            "call_module",
+            self.conv_node.target,
+            (load_arg(self.conv_node.args[0]),),
+            {},
+        )
 
 
 # turn foo.bar -> ['foo', 'bar']
 def _parent_name(target):
-    r = target.rsplit('.', 1)
+    r = target.rsplit(".", 1)
     if len(r) == 1:
-        return '', r[0]
+        return "", r[0]
     else:
         return r[0], r[1]
 
 
-
 class DefaultQuant(MinMaxObserver):
     def quantize(self, input):
         assert self.all_tensors
         scale, zeropoint = self.scale_zeropoint()
-        return torch.quantize_per_tensor(Proxy(input), scale, zeropoint, torch.quint8).node
+        return torch.quantize_per_tensor(
+            Proxy(input), scale, zeropoint, torch.quint8
+        ).node
+
 
 def matches(modules, node, pattern, max_uses=sys.maxsize):
     if isinstance(pattern, tuple):
@@ -168,12 +222,12 @@ def matches(modules, node, pattern, max_uses=sys.maxsize):
         return False
 
     if isinstance(self_match, type) and issubclass(self_match, torch.nn.Module):
-        if node.op != 'call_module':
+        if node.op != "call_module":
             return False
         if not isinstance(modules[node.target], self_match):
             return False
     elif callable(self_match):
-        if node.op != 'call_function' or node.target is not self_match:
+        if node.op != "call_function" or node.target is not self_match:
             return False
     elif node.target != self_match:
         return False
@@ -184,11 +238,16 @@ def matches(modules, node, pattern, max_uses=sys.maxsize):
     if len(arg_matches) != len(node.args):
         return False
 
-    return all(matches(modules, node, arg_match, max_uses=1) for node, arg_match in zip(node.args, arg_matches))
+    return all(
+        matches(modules, node, arg_match, max_uses=1)
+        for node, arg_match in zip(node.args, arg_matches)
+    )
 
 
 class Quantizer:
-    def __init__(self, mod, patterns=DEFAULT_QUANTIZATION_PATTERNS, quant_ctor=DefaultQuant):
+    def __init__(
+        self, mod, patterns=_DEFAULT_QUANTIZATION_PATTERNS, quant_ctor=DefaultQuant
+    ):
         self.root = mod
         self.graph = mod.graph
         self.quant_ctor = quant_ctor
@@ -204,8 +263,6 @@ class Quantizer:
         # initialize an quant_ctor object for each
         self.quants = self._find_quants(quant_ctor)
 
-
-
     def observe(self, args):
         # most of this function is just an interpreter for the graph
         # it would be possible to put this in some abstraction, but
@@ -219,21 +276,22 @@ class Quantizer:
         def load_arg(a):
             return map_arg(a, lambda node: env[node.name])
 
-        output_node : Optional[Node] = None
         for node in self.graph.nodes:
-            if node.op == 'placeholder':
+            if node.op == "placeholder":
                 result = next(args_iter)
-            elif node.op == 'get_attr':
+            elif node.op == "get_attr":
                 result = self.state_dict[node.target]
-            elif node.op == 'call_function':
+            elif node.op == "call_function":
                 result = node.target(*load_arg(node.args), **load_arg(node.kwargs))
-            elif node.op == 'call_method':
+            elif node.op == "call_method":
                 self_obj, *args = load_arg(node.args)
                 kwargs = load_arg(node.kwargs)
                 result = getattr(self_obj, node.target)(*args, **kwargs)
-            elif node.op == 'call_module':
-                result = self.modules[node.target](*load_arg(node.args), **load_arg(node.kwargs))
-            elif node.op == 'output':
+            elif node.op == "call_module":
+                result = self.modules[node.target](
+                    *load_arg(node.args), **load_arg(node.kwargs)
+                )
+            elif node.op == "output":
                 return load_arg(node.args[0])
 
             env[node.name] = result
@@ -243,7 +301,7 @@ class Quantizer:
             if node.name in self.quants:
                 self.quants[node.name].observe(node, env)
 
-        raise RuntimeError('Graph had no output node!')
+        raise RuntimeError("Graph had no output node!")
 
     def quantize(self):
         self.quantized_graph = Graph()
@@ -262,22 +320,25 @@ class Quantizer:
                 return quant_env[n.name]
 
         def copy_recursive(node):
-            def load_or_emit(n):
-                if n.name in env or e.name in quant_env:
-                    return load_arg(n, quantized=False)
-                else:
-                    return copy_recusive(n)
-            r = env[node.name] = self.quantized_graph.node_copy(node, lambda n: load_arg(n, quantized=False))
+            r = env[node.name] = self.quantized_graph.node_copy(
+                node, lambda n: load_arg(n, quantized=False)
+            )
             return r
 
         for node in self.graph.nodes:
             root_node, obj = self.matches.get(node.name, (None, None))
             if root_node is None:
                 # not quantized just copy it
-                env[node.name] = self.quantized_graph.node_copy(node, lambda n: load_arg(n, quantized=False))
+                env[node.name] = self.quantized_graph.node_copy(
+                    node, lambda n: load_arg(n, quantized=False)
+                )
 
             elif root_node is node:
-                r = obj.quantize(self, node, lambda a: map_arg(a, lambda n: load_arg(n, quantized=True)))
+                r = obj.quantize(
+                    self,
+                    node,
+                    lambda a: map_arg(a, lambda n: load_arg(n, quantized=True)),
+                )
                 if r is NotImplemented:
                     # quantizer choose to to quantize the node take the entire match, and just copy it over
                     env[node.name] = copy_recursive(node)
@@ -317,6 +378,7 @@ class Quantizer:
             # say NotImplemented (if for instance, it is an __add__ and the data type is not appropriate)
             if n.name not in quants:
                 quants[n.name] = quant_ctor(self, n)
+
         for node in self.graph.nodes:
             if node.name in self.matches:
                 map_arg(node.args, visit_arg)
