@@ -1,18 +1,23 @@
-import io
+# Owner(s): ["oncall: jit"]
+# ruff: noqa: F841
+
 import os
 import sys
+from typing import Any, Tuple
 
 import torch
 import torch.nn as nn
 
-from typing import Any, Tuple
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
-from torch.testing._internal.jit_utils import JitTestCase, _inline_everything
 from typing import List
+
 from torch import Tensor
+from torch.jit import Future
+from torch.testing._internal.jit_utils import _inline_everything, JitTestCase
+
 
 class TestAsync(JitTestCase):
     def test_async_python(self):
@@ -49,8 +54,7 @@ class TestAsync(JitTestCase):
             futures = torch.jit.annotate(List[Future[List[Tensor]]], [])
             for _ in range(3):
                 future = torch.jit.annotate(
-                    Future[List[Tensor]],
-                    torch.jit.fork(foo, x)
+                    Future[List[Tensor]], torch.jit.fork(foo, x)
                 )
                 futures.append(future)
 
@@ -83,10 +87,10 @@ class TestAsync(JitTestCase):
 
     def test_async_script_capture(self):
         class Mod(torch.jit.ScriptModule):
-            __constants__ = ['const']
+            __constants__ = ["const"]
 
-            def __init__(self):
-                super(Mod, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.const = 42
                 self.param = nn.Parameter(torch.randn(2, 2))
 
@@ -137,7 +141,10 @@ class TestAsync(JitTestCase):
     def test_async_script_no_script_mod(self):
         x = torch.rand(3, 4)
 
-        with self.assertRaisesRegex(RuntimeError, 'cannot call a value'):
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, "cannot call a value", "torch.jit._fork(x"
+        ):
+
             @torch.jit.script
             def wait_script(x):
                 fut = torch.jit._fork(x)
@@ -211,7 +218,7 @@ class TestAsync(JitTestCase):
             lambda x1, x2: torch.jit._wait(torch.jit._fork(foo, x1, x2)),
             lambda x1, x2: torch.jit._wait(torch.jit._fork(foo, x1, x2=x2)),
             lambda x1, x2: torch.jit._wait(torch.jit._fork(foo, x1=x1, x2=x2)),
-            lambda x1, x2: torch.jit._wait(torch.jit._fork(foo, x2=x2, x1=x1))
+            lambda x1, x2: torch.jit._wait(torch.jit._fork(foo, x2=x2, x1=x1)),
         ]:
             for wrapper in [
                 func,
@@ -232,8 +239,8 @@ class TestAsync(JitTestCase):
             return torch.jit._wait(torch.jit._fork(foo, x1=x1, x2=x2))
 
         for wrapper in [
-                foo_script_args,
-                foo_script_kwargs,
+            foo_script_args,
+            foo_script_kwargs,
         ]:
             self.assertEqual(wrapper(x1, x2), y_hat)
             self.assertEqual(wrapper(x1, x2=x2), y_hat)
@@ -243,20 +250,19 @@ class TestAsync(JitTestCase):
     @_inline_everything
     def test_async_script_trace(self):
         class Traced(nn.Module):
-            def __init__(self):
-                super(Traced, self).__init__()
-
             def forward(self, x):
                 return (torch.neg(x), x)
 
         class Mod(torch.jit.ScriptModule):
-            def __init__(self):
-                super(Mod, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 x = torch.rand(3, 3)
                 self.traced = torch.jit.trace(Traced(), (x), _force_outplace=True)
 
             @torch.jit.script_method
-            def forward(self, x: Tensor) -> Tuple[List[Tensor], Tuple[Tensor, Tensor], Tensor]:
+            def forward(
+                self, x: Tensor
+            ) -> Tuple[List[Tensor], Tuple[Tensor, Tensor], Tensor]:
                 future1 = torch.jit._fork(self.traced, x)
                 future2 = torch.jit._fork(torch.neg, x)
 
@@ -271,8 +277,8 @@ class TestAsync(JitTestCase):
                 return (tensor_list, tensor_tuple, tensor_tuple[1])
 
         class TupleCl(nn.Module):
-            def __init__(self):
-                super(TupleCl, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.module = Mod()
 
             def forward(self, x):
@@ -285,10 +291,16 @@ class TestAsync(JitTestCase):
         module = torch.jit.trace(TupleCl(), (x), _force_outplace=True)
 
         # Make sure we have forks
-        self.assertGraphContainsExactly(module.graph, kind='prim::fork', num_kind_nodes=2)
+        self.assertGraphContainsExactly(
+            module.graph, kind="prim::fork", num_kind_nodes=2
+        )
         # Make sure 1 ::neg is in the root graph and 2 ::negs are in the subgraphs
-        self.assertGraphContainsExactly(module.graph, kind='aten::neg', num_kind_nodes=1)
-        self.assertGraphContainsExactly(module.graph, kind='aten::neg', num_kind_nodes=3, consider_subgraphs=True)
+        self.assertGraphContainsExactly(
+            module.graph, kind="aten::neg", num_kind_nodes=1
+        )
+        self.assertGraphContainsExactly(
+            module.graph, kind="aten::neg", num_kind_nodes=3, consider_subgraphs=True
+        )
 
         y = torch.neg(x)
         self.assertEqual(module(x), (y, y, y, y, x, x))
@@ -312,17 +324,23 @@ class TestAsync(JitTestCase):
             return torch.jit._wait(fut)
 
         # no future
-        error_msg = 'The size.*must match the size of tensor'
-        with self.assertRaisesRegex(Exception, error_msg):
+        error_msg = "The size.*must match the size of tensor"
+        with self.assertRaisesRegexWithHighlight(Exception, error_msg, "x.t() + x"):
             foo(x)
 
         # one future
-        with self.assertRaisesRegex(Exception, error_msg):
+        with self.assertRaisesRegexWithHighlight(
+            Exception, error_msg, "torch.jit._fork(foo, x"
+        ):
             wait_script(x)
 
         # two futures with a different error
         x = torch.rand(3, 4, 5)
-        with self.assertRaisesRegex(Exception, 'expects a tensor with <= 2 dimensions'):
+        with self.assertRaisesRegexWithHighlight(
+            Exception,
+            "expects a tensor with <= 2 dimensions",
+            "torch.jit._fork(wait_script, x",
+        ):
             wait_script_nest(x)
 
     def test_async_grad_guard_with_grad(self):
@@ -380,9 +398,15 @@ class TestAsync(JitTestCase):
         x = torch.rand(3, 4)
         self.assertEqual(fn(x), traced(x))
 
-        self.assertGraphContainsExactly(traced.graph, kind='prim::fork', num_kind_nodes=1)
-        self.assertGraphContainsExactly(traced.graph, kind='aten::wait', num_kind_nodes=1)
-        self.assertGraphContainsExactly(traced.graph, kind='aten::neg', num_kind_nodes=2, consider_subgraphs=True)
+        self.assertGraphContainsExactly(
+            traced.graph, kind="prim::fork", num_kind_nodes=1
+        )
+        self.assertGraphContainsExactly(
+            traced.graph, kind="aten::wait", num_kind_nodes=1
+        )
+        self.assertGraphContainsExactly(
+            traced.graph, kind="aten::neg", num_kind_nodes=2, consider_subgraphs=True
+        )
 
     def test_trace_fork_wait_leaking(self):
         my_list = []
@@ -396,9 +420,13 @@ class TestAsync(JitTestCase):
             val = torch.jit._wait(fut)
             return my_list[0]
 
-        with self.assertRaisesRegex(RuntimeError, 'did not have observable data dependence with trace inputs; '
-                                                  'this probably indicates your program cannot be understood '
-                                                  'by the tracer.'):
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError,
+            "did not have observable data dependence with trace inputs; "
+            "this probably indicates your program cannot be understood "
+            "by the tracer.",
+            "",
+        ):
             traced = torch.jit.trace(fn, (torch.rand(3, 4),), check_trace=False)
 
     def test_trace_fork_wait_inline(self):
@@ -412,32 +440,21 @@ class TestAsync(JitTestCase):
 
         traced = torch.jit.trace(fn, (torch.rand(3, 4),))
         torch._C._jit_pass_inline_fork_wait(traced.graph)
-        self.assertGraphContainsExactly(traced.graph, kind='prim::fork', num_kind_nodes=0)
-        self.assertGraphContainsExactly(traced.graph, kind='aten::wait', num_kind_nodes=0)
-        self.assertGraphContainsExactly(traced.graph, kind='aten::add', num_kind_nodes=2)
-
-    def test_trace_fork_wait_inline_onnx(self):
-        def fork_body(x):
-            return torch.neg(x), torch.neg(x)
-
-        class MyMod(torch.nn.Module):
-            def forward(self, x):
-                fut = torch.jit._fork(fork_body, x)
-                val = torch.jit._wait(fut)
-                return val[1]
-
-        # smoke test for ONNX export
-        f = io.BytesIO()
-        torch.onnx.export(MyMod(), (torch.rand(3, 4),), f)
+        self.assertGraphContainsExactly(
+            traced.graph, kind="prim::fork", num_kind_nodes=0
+        )
+        self.assertGraphContainsExactly(
+            traced.graph, kind="aten::wait", num_kind_nodes=0
+        )
+        self.assertGraphContainsExactly(
+            traced.graph, kind="aten::add", num_kind_nodes=2
+        )
 
     def test_trace_fork_wait_list_modulecalls(self):
         def add_one(input):
             return input + torch.ones(input.size())
 
         class TestListFutureModule(nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(self, input):
                 input_list = []
                 for i in range(3):
@@ -451,7 +468,7 @@ class TestAsync(JitTestCase):
                 return fut_list
 
         class TestModuleWrapper(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.list_fut_mod = TestListFutureModule()
 
@@ -469,9 +486,6 @@ class TestAsync(JitTestCase):
             return input + torch.ones(input.size())
 
         class DifferentOutputModule(nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(self, input):
                 fut_res = torch.jit._fork(add_one, (input))
 
@@ -479,7 +493,7 @@ class TestAsync(JitTestCase):
                 return input, fut_res
 
         class TestModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.gen_output = DifferentOutputModule()
 
@@ -491,7 +505,10 @@ class TestAsync(JitTestCase):
         self.checkTrace(TestModule(), (torch.randn(5, 5),))
 
     def test_no_future_subtype_message(self):
-        with self.assertRaisesRegex(RuntimeError, 'Future without a contained type'):
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, "Future without a contained type", ""
+        ):
+
             @torch.jit.script
             def forward(self, x):
                 futs = torch.jit.annotate(List[torch.jit.Future], [])
@@ -500,6 +517,7 @@ class TestAsync(JitTestCase):
         """
         Test that futures subtype each other properly.
         """
+
         # Successful subtyping.
         def returns_int(x: int) -> int:
             return x + x + 1
@@ -513,10 +531,12 @@ class TestAsync(JitTestCase):
             return fut.wait()
 
         # Unsuccessful subtyping.
-        with self.assertRaisesRegex(
-                RuntimeError,
-                r"was annotated as having type Future\[float\] but is actually of type Future\[int\]",
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError,
+            r"was annotated as having type Future\[float\] but is actually of type Future\[int\]",
+            "fut = returns_future_float(x",
         ):
+
             def returns_future_float(x: int) -> torch.jit.Future[float]:
                 return torch.jit._fork(returns_int, (x))
 
@@ -526,8 +546,9 @@ class TestAsync(JitTestCase):
                 return fut.wait()
 
 
-
-if __name__ == '__main__':
-    raise RuntimeError("This test file is not meant to be run directly, use:\n\n"
-                       "\tpython test/test_jit.py TESTNAME\n\n"
-                       "instead.")
+if __name__ == "__main__":
+    raise RuntimeError(
+        "This test file is not meant to be run directly, use:\n\n"
+        "\tpython test/test_jit.py TESTNAME\n\n"
+        "instead."
+    )
