@@ -1,6 +1,9 @@
-import torch
+# Owner(s): ["oncall: jit"]
+
 from typing import List, Tuple
 
+import torch
+
 
 class SubmoduleNoForwardInputs(torch.nn.Module):
     def __init__(self, name):
@@ -46,7 +49,7 @@ class ModuleForwardSingleInput(torch.nn.Module):
         return self.submodule(input)
 
 
-class ModuleDirectFowardSubmodCall(torch.nn.Module):
+class ModuleDirectforwardSubmodCall(torch.nn.Module):
     def __init__(self, name: str, submodule_name: str):
         super().__init__()
         self.name = name
@@ -85,7 +88,7 @@ class SubmoduleForwardTupleInput(torch.nn.Module):
         self.name = name
 
     def forward(self, input: Tuple[int]):
-        input_access = input[0]
+        input_access = input[0]  # noqa: F841
         return (1,)
 
 
@@ -96,7 +99,7 @@ class ModuleForwardTupleInput(torch.nn.Module):
         self.submodule = SubmoduleForwardTupleInput(submodule_name)
 
     def forward(self, input: Tuple[int]):
-        input_access = input[0]
+        input_access = input[0]  # noqa: F841
         return self.submodule((1,))
 
 
