@@ -1,30 +1,44 @@
+# Owner(s): ["oncall: jit"]
+# ruff: noqa: F841
+
 import os
+import re
 import sys
+import types
 import typing
 import typing_extensions
-from typing import List, Dict, Optional, Tuple
+from collections import OrderedDict
+from typing import Dict, List, Optional, Tuple
 
 import torch
+import torch.jit.frontend
 import torch.nn as nn
 from torch import Tensor
 from torch.testing import FileCheck
-from collections import OrderedDict
+
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
-from torch.testing._internal.jit_utils import JitTestCase, _tmp_donotuse_dont_inline_everything
+from torch.testing._internal.jit_utils import (
+    _tmp_donotuse_dont_inline_everything,
+    JitTestCase,
+)
+
+
+if __name__ == "__main__":
+    raise RuntimeError(
+        "This test file is not meant to be run directly, use:\n\n"
+        "\tpython test/test_jit.py TESTNAME\n\n"
+        "instead."
+    )
 
-if __name__ == '__main__':
-    raise RuntimeError("This test file is not meant to be run directly, use:\n\n"
-                       "\tpython test/test_jit.py TESTNAME\n\n"
-                       "instead.")
 
 class TestRecursiveScript(JitTestCase):
     def test_inferred_nonetype(self):
         class M(nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.x = None
 
             def forward(self):
@@ -44,7 +58,7 @@ class TestRecursiveScript(JitTestCase):
 
         class M(torch.nn.Module):
             def __init__(self, fn):
-                super(M, self).__init__()
+                super().__init__()
                 self.fn = fn
 
             def forward(self, x):
@@ -59,7 +73,7 @@ class TestRecursiveScript(JitTestCase):
     def test_python_function_attribute(self):
         class M(torch.nn.Module):
             def __init__(self, fn):
-                super(M, self).__init__()
+                super().__init__()
                 self.fn = fn
 
             def forward(self, x):
@@ -71,23 +85,25 @@ class TestRecursiveScript(JitTestCase):
 
     def test_failed_function_compilation(self):
         def fn(x):
-            return i_dont_exist
+            return i_dont_exist  # noqa: F821
 
         class M(torch.nn.Module):
             def __init__(self, fn):
-                super(M, self).__init__()
+                super().__init__()
                 self.fn = fn
 
             def forward(self, x):
                 return self.fn(x)
 
         m = M(fn)
-        with self.assertRaisesRegex(RuntimeError, "failed to compile"):
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, "failed to compile", "i_dont_exist"
+        ):
             torch.jit.script(m)
 
     def test_init_error(self):
         class M(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 self.x = 2
 
             def forward(self):
@@ -114,18 +130,18 @@ class TestRecursiveScript(JitTestCase):
 
         # sm1 was created while m had training = True
         self.assertTrue(sm1.training)
-        self.assertEqual(sm1.training, sm1._c.getattr('training'))
+        self.assertEqual(sm1.training, sm1._c.getattr("training"))
         self.assertEqual(sm1(), 2)
 
         # sm2 was created after m was eval'ed
         self.assertFalse(sm2.training)
-        self.assertEqual(sm2.training, sm2._c.getattr('training'))
+        self.assertEqual(sm2.training, sm2._c.getattr("training"))
         self.assertEqual(sm2(), 0)
 
     def test_module_name(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
-                super(MyModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.x = 2
 
             def forward(self, t):
@@ -160,9 +176,9 @@ class TestRecursiveScript(JitTestCase):
 
     def test_constants_with_final(self):
         class M1(torch.nn.Module):
-            x : torch.jit.Final[int]
+            x: torch.jit.Final[int]
 
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.x = 2
 
@@ -172,9 +188,9 @@ class TestRecursiveScript(JitTestCase):
         self.checkModule(M1(), (torch.randn(2, 2),))
 
         class M2(torch.nn.Module):
-            x : typing_extensions.Final[int]
+            x: typing_extensions.Final[int]
 
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.x = 2
 
@@ -183,34 +199,33 @@ class TestRecursiveScript(JitTestCase):
 
         self.checkModule(M2(), (torch.randn(2, 2),))
 
-        if sys.version_info[:2] >= (3, 8):
-            class M3(torch.nn.Module):
-                x : typing.Final[int]
+        class M3(torch.nn.Module):
+            x: typing.Final[int]
 
-                def __init__(self):
-                    super().__init__()
-                    self.x = 2
+            def __init__(self) -> None:
+                super().__init__()
+                self.x = 2
 
-                def forward(self, t):
-                    return t + self.x
+            def forward(self, t):
+                return t + self.x
 
-            self.checkModule(M3(), (torch.randn(2, 2),))
+        self.checkModule(M3(), (torch.randn(2, 2),))
 
     def test_ignore_class(self):
         @torch.jit.ignore
-        class MyScriptClass(object):
+        class MyScriptClass:
             def unscriptable(self):
                 return "a" + 200
 
-
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
-
             def forward(self, x):
                 return MyScriptClass()
 
-        with self.assertRaisesRegex(torch.jit.frontend.FrontendError, "Cannot instantiate class"):
+        with self.assertRaisesRegexWithHighlight(
+            torch.jit.frontend.FrontendError,
+            "Cannot instantiate class",
+            "MyScriptClass",
+        ):
             t = torch.jit.script(TestModule())
 
     def test_method_call(self):
@@ -230,8 +245,8 @@ class TestRecursiveScript(JitTestCase):
                 return x
 
         class MyModule(nn.Module):
-            def __init__(self):
-                super(MyModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.conv = nn.Conv2d(10, 10, 3)
                 self.lin = nn.Linear(10, 10)
                 self.sub = Submodule()
@@ -245,13 +260,13 @@ class TestRecursiveScript(JitTestCase):
             print(m)
 
         f = FileCheck()
-        f.check('MyModule')
-        f.check('Conv2d')
-        f.check('Linear')
-        f.check('Submodule')
+        f.check("MyModule")
+        f.check("Conv2d")
+        f.check("Linear")
+        f.check("Submodule")
         f.run(out[0])
 
-        self.assertEqual(m.original_name, 'MyModule')
+        self.assertEqual(m.original_name, "MyModule")
 
     def test_dir(self):
         def test_module_dir(mod):
@@ -259,16 +274,25 @@ class TestRecursiveScript(JitTestCase):
             scripted_mod = torch.jit.script(mod)
             dir_scripted = set(dir(scripted_mod))
             # set not currently copied over
-            ignore_set = ["training", "__delitem__", "__setitem__", "clear", "items",
-                          "keys", "pop", "update", "values"]
+            ignore_set = [
+                "training",
+                "__delitem__",
+                "__setitem__",
+                "clear",
+                "items",
+                "keys",
+                "pop",
+                "update",
+                "values",
+            ]
             for attr in dir_set:
                 if attr in ignore_set:
                     continue
                 self.assertTrue(attr in dir_scripted, attr)
 
         class MyModule(nn.Module):
-            def __init__(self):
-                super(MyModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.conv = nn.Conv2d(10, 10, 3)
                 self.lin = nn.Linear(10, 10)
 
@@ -282,24 +306,22 @@ class TestRecursiveScript(JitTestCase):
         linear = nn.Linear(10, 10)
 
         test_module_dir(nn.Sequential(conv, linear))
-        test_module_dir(nn.ModuleDict(OrderedDict([("conv", conv), ("linear", linear)])))
+        test_module_dir(
+            nn.ModuleDict(OrderedDict([("conv", conv), ("linear", linear)]))
+        )
 
     def test_class_compile(self):
         def other_fn(a: int, b: Tensor) -> Tensor:
             return a * b
 
-        class B(object):
+        class B:
             def __init__(self, x):
                 self.x = 2
 
             def helper(self, a):
                 return self.x + a + other_fn(self.x, a)
 
-
         class N(torch.nn.Module):
-            def __init__(self):
-                super(N, self).__init__()
-
             def forward(self, x):
                 b = B(x)
                 return b.helper(x)
@@ -340,15 +362,12 @@ class TestRecursiveScript(JitTestCase):
             return c(x)
 
         class Submodule(torch.nn.Module):
-            def __init__(self):
-                super(Submodule, self).__init__()
-
             def forward(self, x):
                 return b(x)
 
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.submodule = Submodule()
 
             def some_method(self, y):
@@ -382,9 +401,9 @@ class TestRecursiveScript(JitTestCase):
         self.assertEqual(a_script_fn(t, t, t), t + t + t)
 
     def test_error_stack_class(self):
-        class X(object):
+        class X:
             def bad_fn(self):
-                import pdb  # noqa
+                import pdb  # noqa: F401
 
         def fn(x) -> X:
             return X(10)
@@ -398,9 +417,9 @@ class TestRecursiveScript(JitTestCase):
             checker.run(str(e))
 
     def test_error_stack_annotation(self):
-        class X(object):
+        class X:
             def bad_fn(self):
-                import pdb  # noqa
+                import pdb  # noqa: F401
 
         def fn(x) -> X:
             return X(10)
@@ -416,10 +435,10 @@ class TestRecursiveScript(JitTestCase):
 
     def test_module_basic(self):
         class Other(torch.nn.Module):
-            __constants__ = ['x']
+            __constants__ = ["x"]
 
             def __init__(self, x):
-                super(Other, self).__init__()
+                super().__init__()
                 self.x = x
                 self.param = torch.nn.Parameter(torch.ones(2, 2))
 
@@ -431,10 +450,9 @@ class TestRecursiveScript(JitTestCase):
             def forward(self, t):
                 return t + self.x + self.param
 
-
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.other = Other(200)
 
             def forward(self, t):
@@ -444,10 +462,10 @@ class TestRecursiveScript(JitTestCase):
 
     def test_module_function_export(self):
         class Other(torch.nn.Module):
-            __constants__ = ['x']
+            __constants__ = ["x"]
 
             def __init__(self, x):
-                super(Other, self).__init__()
+                super().__init__()
                 self.x = x
                 self.param = torch.nn.Parameter(torch.ones(2, 2))
 
@@ -458,10 +476,9 @@ class TestRecursiveScript(JitTestCase):
             def forward(self, t):
                 return t + self.x + self.param
 
-
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.other = Other(200)
 
             def forward(self, t):
@@ -475,12 +492,10 @@ class TestRecursiveScript(JitTestCase):
                 return x + 10
 
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.sequential = nn.Sequential(
-                    Inner(),
-                    Inner(),
-                    nn.Sequential(Inner(), Inner())
+                    Inner(), Inner(), nn.Sequential(Inner(), Inner())
                 )
                 self.module_list = nn.ModuleList([Inner(), Inner()])
 
@@ -510,18 +525,20 @@ class TestRecursiveScript(JitTestCase):
                 return nn.ReLU()
 
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 shared = SeluButReluWhenScripted()
                 self.sequential = nn.Sequential(
                     SeluButReluWhenScripted(),
                     SeluButReluWhenScripted(),
-                    nn.Sequential(SeluButReluWhenScripted(), shared, SeluButReluWhenScripted()),
+                    nn.Sequential(
+                        SeluButReluWhenScripted(), shared, SeluButReluWhenScripted()
+                    ),
                     shared,
                 )
-                self.module_list = nn.ModuleList([SeluButReluWhenScripted(),
-                                                  shared,
-                                                  SeluButReluWhenScripted()])
+                self.module_list = nn.ModuleList(
+                    [SeluButReluWhenScripted(), shared, SeluButReluWhenScripted()]
+                )
 
             def forward(self, x):
                 for mod in self.module_list:
@@ -545,21 +562,57 @@ class TestRecursiveScript(JitTestCase):
 
         sm = torch.jit.script(p)
 
+    def test_prepare_scriptable_escape_hatch(self):
+        class NonJitableClass:
+            def __call__(self, int1, int2, *args):
+                total = int1 + int2
+                for arg in args:
+                    total += arg
+                return total
+
+        obj = NonJitableClass()
+
+        self.assertEqual(obj(1, 2), 3)
+        self.assertEqual(obj(1, 2, 3, 4), 10)
+        with self.assertRaisesRegex(
+            torch.jit.frontend.NotSupportedError,
+            expected_regex="can't take variable number of arguments",
+        ):
+            torch.jit.script(obj)
+
+        def escape_hatch(int1: int, int2: int) -> int:
+            return int1 + int2
+
+        class NonJitableClassWithEscapeHatch(NonJitableClass):
+            def __prepare_scriptable__(self):
+                return escape_hatch
+
+        jit_obj = torch.jit.script(NonJitableClassWithEscapeHatch())
+
+        self.assertEqual(jit_obj(1, 2), 3)
+        with self.assertRaisesRegex(
+            RuntimeError,
+            expected_regex=re.escape(
+                "expected at most 2 argument(s) but received 4 argument(s)"
+            ),
+        ):
+            jit_obj(1, 2, 3, 4)
+
     def test_attributes(self):
         @torch.jit.script
-        class Inner2(object):
-            def __init__(self):
+        class Inner2:
+            def __init__(self) -> None:
                 self.b = "a string"
 
         @torch.jit.script
-        class Foo(object):
-            def __init__(self):
+        class Foo:
+            def __init__(self) -> None:
                 self.a = 4
                 self.inner = Inner2()
 
         @torch.jit.script
-        class SFoo(object):
-            def __init__(self):
+        class SFoo:
+            def __init__(self) -> None:
                 self.a = 4
                 self.inner = Inner2()
 
@@ -571,27 +624,26 @@ class TestRecursiveScript(JitTestCase):
             def __getstate__(self):
                 return (self.a, self.inner)
 
-
         untyped_values = (
-            ('my_dict', {"I": "am", "a test": "test"}),
-            ('my_float', 2.3),
-            ('my_int', 99),
-            ('my_bool', False),
-            ('my_tuple', (1, 2, 3, 4)),
-            ('my_list', [(1, 2), (3, 4)]),
+            ("my_dict", {"I": "am", "a test": "test"}),
+            ("my_float", 2.3),
+            ("my_int", 99),
+            ("my_bool", False),
+            ("my_tuple", (1, 2, 3, 4)),
+            ("my_list", [(1, 2), (3, 4)]),
             # ('my_tensor', torch.randn(2, 2)),
-            ('my_int_list', [1, 2, 3, 4]),
+            ("my_int_list", [1, 2, 3, 4]),
             # ('my_tensor_list', [torch.ones(2, 2) + i for i in range(4)]),
-            ('my_bool_list', [True, True, False, True]),
-            ('my_float_list', [1., 2., 3., 4.]),
-            ('my_str_list', ['hello', 'bye']),
+            ("my_bool_list", [True, True, False, True]),
+            ("my_float_list", [1.0, 2.0, 3.0, 4.0]),
+            ("my_str_list", ["hello", "bye"]),
         )
         typed_values = (
-            ('my_empty_list', []),
-            ('my_empty_dict', {}),
-            ('my_none', None),
-            ('my_object', Foo()),
-            ('my_object2', SFoo()),
+            ("my_empty_list", []),
+            ("my_empty_dict", {}),
+            ("my_none", None),
+            ("my_object", Foo()),
+            ("my_object2", SFoo()),
         )
 
         class M(torch.nn.Module):
@@ -601,9 +653,6 @@ class TestRecursiveScript(JitTestCase):
             # my_empty_dict : Dict[str, int]
             # my_none : Optional[int]
 
-            def __init__(self):
-                super(M, self).__init__()
-
             def forward(self, x):
                 return (
                     self.my_dict,
@@ -635,11 +684,11 @@ class TestRecursiveScript(JitTestCase):
         # since there's no string frontend for Python classes (so the `define`)
         # trick doesn't work.
         M.__annotations__ = {
-            'my_empty_list': List[int],
-            'my_empty_dict': Dict[str, int],
-            'my_none': Optional[int],
-            'my_object': Foo,
-            'my_object2': SFoo,
+            "my_empty_list": List[int],
+            "my_empty_dict": Dict[str, int],
+            "my_none": Optional[int],
+            "my_object": Foo,
+            "my_object2": SFoo,
         }
 
         m = M()
@@ -651,7 +700,7 @@ class TestRecursiveScript(JitTestCase):
     def test_function_attribute_in_submodule(self):
         class N(nn.Module):
             def __init__(self, norm):
-                super(N, self).__init__()
+                super().__init__()
                 self.activation = torch.nn.functional.relu
                 self.norm = norm
 
@@ -661,8 +710,8 @@ class TestRecursiveScript(JitTestCase):
                 return output
 
         class M(nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 encoder_norm = nn.ReLU()
                 self.encoder = N(encoder_norm)
 
@@ -670,7 +719,7 @@ class TestRecursiveScript(JitTestCase):
                 return self.encoder(x)
 
         m = M()
-        self.checkModule(m, (torch.randn(5, 5), ))
+        self.checkModule(m, (torch.randn(5, 5),))
 
     def test_inner_traced_module(self):
         class Dummy(nn.Module):
@@ -679,7 +728,7 @@ class TestRecursiveScript(JitTestCase):
 
         class Model(nn.Module):
             def __init__(self, dummies):
-                super(Model, self).__init__()
+                super().__init__()
                 self._dummies = dummies
 
             def forward(self, x):
@@ -691,12 +740,13 @@ class TestRecursiveScript(JitTestCase):
         dummy = torch.jit.trace(Dummy(), torch.randn(1, 2))
         dummies = nn.ModuleList([dummy])
         model = Model(dummies)
-        self.checkModule(model, (torch.rand(5, 5), ))
+        self.checkModule(model, (torch.rand(5, 5),))
 
     def test_script_loaded_module(self):
         """
         Test that we can hold a loaded ScriptModule as a submodule.
         """
+
         class Dummy(nn.Module):
             def forward(self, x):
                 return x
@@ -705,19 +755,19 @@ class TestRecursiveScript(JitTestCase):
         dummy = self.getExportImportCopy(dummy)
 
         class ContainsLoaded(torch.nn.Module):
-            def __init__(self):
-                super(ContainsLoaded, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.encoder = dummy
 
             def forward(self, input):
                 return self.encoder(input)
 
-        self.checkModule(ContainsLoaded(), (torch.rand(2, 3), ))
+        self.checkModule(ContainsLoaded(), (torch.rand(2, 3),))
 
     def test_optional_module(self):
         class Dummy(nn.Module):
-            def __init__(self):
-                super(Dummy, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.foo = nn.Linear(2, 2)
 
             def forward(self, x):
@@ -729,3 +779,23 @@ class TestRecursiveScript(JitTestCase):
         self.checkModule(mod, (torch.rand(2, 2),))
         mod.foo = None
         self.checkModule(mod, (torch.rand(2, 2),))
+
+    def test_override_instance_method_ignore(self):
+        class M(torch.nn.Module):
+            @torch.jit.ignore
+            def i_am_ignored(self):
+                return "old"
+
+        m = M()
+
+        # Override the ignored method by binding a new method to this instance.
+        @torch.jit.ignore
+        def i_am_ignored(self):
+            return "new"
+
+        m.i_am_ignored = types.MethodType(i_am_ignored, m)
+        self.assertEqual(m.i_am_ignored(), "new")
+
+        # ScriptModule should correctly reflect the override.
+        s = torch.jit.script(m)
+        self.assertEqual(s.i_am_ignored(), "new")
