@@ -1,9 +1,12 @@
+# Owner(s): ["oncall: jit"]
+# ruff: noqa: F841
+
 import os
 import sys
-
 from typing import Any, List
 
 import torch
+from torch.testing._internal.common_utils import skipIfTorchDynamo
 from torch.testing._internal.jit_utils import JitTestCase, make_global
 
 
@@ -29,8 +32,9 @@ class TestWith(JitTestCase):
         Check that with statements that use the 'as' keyword to bind expressions
         to targets work as expected.
         """
+
         @torch.jit.script
-        class Context(object):
+        class Context:
             """
             This class implements a basic context manager interface for use in
             the unit tests. Unlike Context, the stateful part of this class
@@ -45,8 +49,9 @@ class TestWith(JitTestCase):
                 self.count.add_(0.3)
                 return self.count
 
-            def __exit__(self, type: Any, value: Any, tb: Any):
+            def __exit__(self, type: Any, value: Any, tb: Any) -> bool:
                 self.count.sub_(0.3)
+                return True
 
         make_global(Context)
 
@@ -185,8 +190,9 @@ class TestWith(JitTestCase):
         Check that with statements that do not use the 'as' keyword to bind expressions
         to targets work as expected.
         """
+
         @torch.jit.script
-        class Context(object):
+        class Context:
             """
             This class implements a basic context manager interface for use in
             the unit tests. Unlike Context, the stateful part of this class
@@ -341,8 +347,9 @@ class TestWith(JitTestCase):
         Check that exceptions thrown in the bodies of with-statements are
         handled correctly.
         """
+
         @torch.jit.script
-        class Context(object):
+        class Context:
             """
             This class implements a basic context manager interface for use in
             the unit tests. Unlike Context, the stateful part of this class
@@ -364,7 +371,7 @@ class TestWith(JitTestCase):
 
         @torch.jit.script
         def method_that_raises() -> torch.Tensor:
-            raise Exception("raised exception")
+            raise Exception("raised exception")  # noqa: TRY002
 
         @torch.jit.script
         def test_exception(x: torch.Tensor, c: Context) -> torch.Tensor:
@@ -412,15 +419,21 @@ class TestWith(JitTestCase):
         # checkScript and checkScriptRaisesRegex cannot be used because the string frontend will
         # not compile class types (of which Context, the context manager being used for this test
         # is one).
-        with self.assertRaisesRegex(Exception, r"raised exception"):
+        with self.assertRaisesRegexWithHighlight(
+            Exception, r"raised exception", 'raise Exception("raised exception'
+        ):
             test_exception(torch.randn(2), c)
         self.assertEqual(c.count, 1)
 
-        with self.assertRaisesRegex(Exception, r"raised exception"):
+        with self.assertRaisesRegexWithHighlight(
+            Exception, r"raised exception", 'raise Exception("raised exception'
+        ):
             test_exception_nested(torch.randn(2), c)
         self.assertEqual(c.count, 1)
 
-        with self.assertRaisesRegex(Exception, r"raised exception"):
+        with self.assertRaisesRegexWithHighlight(
+            Exception, r"raised exception", 'raise Exception("raised exception'
+        ):
             test_exception_fn_call(torch.randn(2), c)
         self.assertEqual(c.count, 1)
 
@@ -430,51 +443,51 @@ class TestWith(JitTestCase):
         """
 
         @torch.jit.script
-        class NoEnterNoExit(object):
+        class NoEnterNoExit:
             """
             This class is missing __enter__ and __exit__ methods.
             """
 
-            def __init__(self):
+            def __init__(self) -> None:
                 self.count = 1
 
         @torch.jit.script
-        class BadEnter(object):
+        class BadEnter:
             """
-            This class is has an __enter__ method with an incorrect signature.
+            This class has an __enter__ method with an incorrect signature.
             """
 
-            def __init__(self):
+            def __init__(self) -> None:
                 self.count = 1
 
-            def __enter__(self, incr: int):
+            def __enter__(self, incr: int):  # noqa: PLE0302
                 self.count += incr
 
             def __exit__(self, type: Any, value: Any, tb: Any):
                 pass
 
         @torch.jit.script
-        class BadExit(object):
+        class BadExit:
             """
-            This class is has an __exit__ method with an incorrect signature.
+            This class has an __exit__ method with an incorrect signature.
             """
 
-            def __init__(self):
+            def __init__(self) -> None:
                 self.count = 1
 
             def __enter__(self):
                 self.count += 1
 
-            def __exit__(self, type: Any, value: Any):
+            def __exit__(self, type: Any, value: Any):  # noqa: PLE0302
                 pass
 
         @torch.jit.script
-        class ExitIncorrectTypes(object):
+        class ExitIncorrectTypes:
             """
-            This class is has an __exit__ method with unsupported argument types.
+            This class has an __exit__ method with unsupported argument types.
             """
 
-            def __init__(self):
+            def __init__(self) -> None:
                 self.count = 1
 
             def __enter__(self):
@@ -483,54 +496,67 @@ class TestWith(JitTestCase):
             def __exit__(self, type: Any, value: int, tb: int):
                 pass
 
-        def test_no_enter_no_exit(x: torch.Tensor, c: NoEnterNoExit) -> torch.Tensor:
-            with c as _:
+        def test_no_enter_no_exit(x: torch.Tensor, cm: NoEnterNoExit) -> torch.Tensor:
+            with cm as _:
                 pass
 
             return x
 
-        def test_bad_enter(x: torch.Tensor, c: BadEnter) -> torch.Tensor:
-            with c as _:
+        def test_bad_enter(x: torch.Tensor, cm: BadEnter) -> torch.Tensor:
+            with cm as _:
                 pass
 
             return x
 
-        def test_bad_exit(x: torch.Tensor, c: BadExit) -> torch.Tensor:
-            with c as _:
+        def test_bad_exit(x: torch.Tensor, cm: BadExit) -> torch.Tensor:
+            with cm as _:
                 pass
 
             return x
 
-        def test_exit_incorrect_types(x: torch.Tensor, c: ExitIncorrectTypes) -> torch.Tensor:
-            with c as _:
+        def test_exit_incorrect_types(
+            x: torch.Tensor, cm: ExitIncorrectTypes
+        ) -> torch.Tensor:
+            with cm as _:
                 pass
 
             return x
 
+        def test_enter_without_object():
+            with "not_object" as obj:
+                pass
+
         test_tensor = torch.randn(5, dtype=torch.double)
 
-        with self.assertRaisesRegex(
-            RuntimeError, r"does not define __enter__ and __exit__ methods"
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, r"does not define __enter__ and __exit__ methods", "cm"
         ):
             self.checkScript(test_no_enter_no_exit, (test_tensor, NoEnterNoExit()))
 
-        with self.assertRaisesRegex(
-            RuntimeError, r"__enter__ must have only one argument and one return value"
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError,
+            r"__enter__ must have only one argument and one return value",
+            "cm",
         ):
             self.checkScript(test_bad_enter, (test_tensor, BadEnter()))
 
-        with self.assertRaisesRegex(
-            RuntimeError, r"__exit__ must have four arguments and no return value"
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, r"__exit__ must have four arguments", "cm"
         ):
             self.checkScript(test_bad_exit, (test_tensor, BadExit()))
 
-        with self.assertRaisesRegex(
-            RuntimeError, r"argument 2 of __exit__ must have Any type"
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, r"argument 2 of __exit__ must have Any type", "cm"
         ):
             self.checkScript(
                 test_exit_incorrect_types, (test_tensor, ExitIncorrectTypes())
             )
 
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError, r"must return an object", '"not_object"'
+        ):
+            self.checkScript(test_enter_without_object, ())
+
     def test_with_no_grad(self):
         """
         Check that torch.no_grad() works. Most of these are adapted from
@@ -570,9 +596,6 @@ class TestWith(JitTestCase):
         # Check that @torch.jit.ignored functions respect no_grad when it is
         # called in JIT mode.
         class NoGradModule(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             @torch.jit.ignore
             def adder(self, x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
                 w = x + y
@@ -589,11 +612,13 @@ class TestWith(JitTestCase):
 
         self.assertFalse(w.requires_grad)
 
+    @skipIfTorchDynamo("Torchdynamo cannot correctly handle profiler.profile calls")
     def test_with_record_function(self):
         """
         Check that torch.autograd.profiler.record_function context manager is
         torchscriptable.
         """
+
         def with_rf(x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
             with torch.autograd.profiler.record_function("foo"):
                 # Nested record_function.
@@ -611,7 +636,7 @@ class TestWith(JitTestCase):
         function_events = p.function_events
         # Event with name "foo" should be recorded.
         rf_events = [evt for evt in function_events if evt.name == "foo"]
-        self.assertTrue(len(rf_events), 1)
+        self.assertEqual(len(rf_events), 1)
         rf_event = rf_events[0]
         child_events = rf_event.cpu_children
         # Ensure we find nested record_function event
