@@ -1,25 +1,40 @@
 #pragma once
 
+#include <ATen/CachedTensorUtils.h>
+#include <ATen/LegacyBatchedTensorImpl.h>
+#include <ATen/TensorOperators.h>
+#include <torch/csrc/Export.h>
 #include <torch/csrc/autograd/function.h>
-#include <torch/csrc/autograd/variable.h>
 #include <torch/csrc/autograd/utils/grad_layout_contract.h>
-#include <torch/csrc/WindowsTorchApiMacro.h>
-#include <ATen/BatchedTensorImpl.h>
+#include <torch/csrc/autograd/variable.h>
+
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#else
+#include <ATen/ops/_sparse_coo_tensor_unsafe.h>
+#endif
 
 #include <mutex>
 
-namespace torch { namespace autograd {
+namespace torch::autograd {
 
-#define CHECK_RESULT(RESULT, VAR) \
-  if (!(RESULT.is_sparse() || VAR.is_sparse())) { \
-    if (!utils::obeys_layout_contract(RESULT, VAR)) { \
-      TORCH_WARN_ONCE("grad and param do not obey the gradient layout contract. " \
-                      "This is not an error, but may impair performance.\n" \
-                      "grad.sizes() = ", RESULT.sizes(), \
-                      ", strides() = ", RESULT.strides(), "\n", \
-                      "param.sizes() = ", VAR.sizes(), \
-                      ", strides() = ", VAR.strides()); \
-    } \
+#define CHECK_RESULT(RESULT, VAR)                                          \
+  if (!(RESULT.is_sparse() || VAR.is_sparse() || RESULT.is_sparse_csr() || \
+        VAR.is_sparse_csr())) {                                            \
+    if (!utils::obeys_layout_contract(RESULT, VAR)) {                      \
+      TORCH_WARN_ONCE(                                                     \
+          "grad and param do not obey the gradient layout contract. "      \
+          "This is not an error, but may impair performance.\n"            \
+          "grad.sizes() = ",                                               \
+          RESULT.sizes(),                                                  \
+          ", strides() = ",                                                \
+          RESULT.strides(),                                                \
+          "\n",                                                            \
+          "param.sizes() = ",                                              \
+          VAR.sizes(),                                                     \
+          ", strides() = ",                                                \
+          VAR.strides());                                                  \
+    }                                                                      \
   }
 
 struct TORCH_API AccumulateGrad : public Node {
@@ -27,13 +42,20 @@ struct TORCH_API AccumulateGrad : public Node {
 
   variable_list apply(variable_list&& grads) override;
 
-  static at::Tensor callHooks(
-      const Variable& variable,
-      at::Tensor new_grad) {
-    for (auto& hook : impl::hooks(variable)) {
-      new_grad = (*hook)({new_grad})[0];
-    }
-    return new_grad;
+  std::vector<std::unique_ptr<FunctionPreHook>>& tensor_pre_hooks() noexcept
+      override {
+    // NB: Since the AccumulateGrad Node is only a weak ref from the Tensor,
+    //     it can be destroyed even though the Tensor is still alive (contrary
+    //     to all other Nodes). So we must lazily read the Tensor hooks here.
+    return impl::hooks(variable);
+  }
+
+  std::unique_ptr<PostAccumulateGradHook>& tensor_post_acc_grad_hooks()
+      const noexcept override {
+    // NB: Since the AccumulateGrad Node is only a weak ref from the Tensor,
+    //     it can be destroyed even though the Tensor is still alive (contrary
+    //     to all other Nodes). So we must lazily read the Tensor hooks here.
+    return impl::post_acc_grad_hooks(variable);
   }
 
   // Given a variable with its current grad as variable_grad, accumulates
@@ -57,20 +79,20 @@ struct TORCH_API AccumulateGrad : public Node {
   //
   // If variable's grad already exists (variable_grad.defined()), new_grad must
   // be added to variable_grad.  If we aren't setting up for double backward
-  // (!GradMode::is_enabled()), AccumulateGrad performs "variable_grad += new_grad"
-  // in-place, which keeps variable_grad's layout. We assume (hope) variable_grad
-  // was created obeying (1) or (2) at some point in the past.
+  // (!GradMode::is_enabled()), AccumulateGrad performs "variable_grad +=
+  // new_grad" in-place, which keeps variable_grad's layout. We assume (hope)
+  // variable_grad was created obeying (1) or (2) at some point in the past.
   //
   // If we are setting up for double backward, AccumulateGrad updates the grad
-  // out-of-place via "variable_grad + new_grad."  TensorIterator operator+ decides
-  // result's layout.  Typically TensorIterator matches strides of the first arg,
-  // so we once again assume (hope) variable_grad was originally created obeying
-  // (1) or (2).
+  // out-of-place via "variable_grad + new_grad."  TensorIterator operator+
+  // decides result's layout.  Typically TensorIterator matches strides of the
+  // first arg, so we once again assume (hope) variable_grad was originally
+  // created obeying (1) or (2).
   //
-  // AccumulateGrad does not enforce the contract with 100% certainty.  Examples:
+  // AccumulateGrad does not enforce the contract with 100% certainty. Examples:
   //  - If a user manually permutes a param or its grad, then runs a fwd+bwd,
-  //    variable_grad += new_grad keeps variable_grad's layout without rechecking
-  //    the contract.
+  //    variable_grad += new_grad keeps variable_grad's layout without
+  //    rechecking the contract.
   //  - If TensorIterator changes its corner cases about operator+'s result
   //    (for example, giving more or less priority to channels_last inputs, see
   //    https://github.com/pytorch/pytorch/pull/37968) the result may not obey.
@@ -81,7 +103,7 @@ struct TORCH_API AccumulateGrad : public Node {
 
   // variable: the variable whose grad we're accumulating.
   // variable_grad: the current grad for the variable.
-  // new_grad: new grad we want to acummulate for the variable.
+  // new_grad: new grad we want to accumulate for the variable.
   // num_expected_refs: the number of refs we expect to hold internally
   //                    such that it is safe to avoid cloning the grad
   //                    if use_count() of the grad is less than or equal
@@ -97,16 +119,19 @@ struct TORCH_API AccumulateGrad : public Node {
       size_t num_expected_refs,
       const T& update_grad) {
     if (!variable_grad.defined()) {
-      if (!GradMode::is_enabled() &&
-          !new_grad.is_sparse() &&
-          new_grad.use_count() <= num_expected_refs &&
-          ((!new_grad.is_mkldnn() && utils::obeys_layout_contract(new_grad, variable))
-           || new_grad.is_mkldnn())){
+      if (!GradMode::is_enabled() && !new_grad.is_sparse() &&
+          !new_grad.is_sparse_csr() &&
+          !(variable.is_sparse_csr() && new_grad.layout() == at::kStrided) &&
+          at::caching::adjusted_use_count(new_grad) <= num_expected_refs &&
+          (new_grad.is_mkldnn() ||
+           utils::obeys_layout_contract(new_grad, variable))) {
         // we aren't setting up for double-backward
         // not sparse
         // no other user-visible tensor references new_grad
-        // new_grad obeys the "Gradient Layout Contract"
-        // Under these conditions, we can steal new_grad without a deep copy.
+        // new_grad obeys the "Gradient Layout Contract", there has a special
+        // case, For MKLDNN tensor, which is a opaque tensor, assuming it obeys
+        // layout_contract. Under these conditions, we can steal new_grad
+        // without a deep copy.
         update_grad(new_grad.detach());
       } else if (
           !GradMode::is_enabled() && new_grad.is_sparse() &&
@@ -122,17 +147,25 @@ struct TORCH_API AccumulateGrad : public Node {
         // shallow copy. We need a shallow copy so that modifying the original
         // grad tensor doesn't modify the grad we accumulate.
         // We only skip clone if indices and values themselves are contiguous
-        // for backward compatiblity reasons. Since without this optimization,
+        // for backward compatibility reasons. Since without this optimization,
         // earlier we would clone the entire SparseTensor which cloned indices
         // and values.
         // For details see https://github.com/pytorch/pytorch/issues/34375.
+
+        // No scenario where we expect this to be true currently
+        TORCH_INTERNAL_ASSERT_DEBUG_ONLY(
+            !at::caching::is_cached_tensor(new_grad._indices()) &&
+            !at::caching::is_cached_tensor(new_grad._values()) &&
+            !at::caching::is_cached_tensor(new_grad));
+
         update_grad(at::_sparse_coo_tensor_unsafe(
             new_grad._indices(),
             new_grad._values(),
             new_grad.sizes(),
             new_grad.options()));
       } else {
-        if (new_grad.is_sparse()) {
+        if (new_grad.is_sparse() || new_grad.is_sparse_csr() ||
+            new_grad.is_nested()) {
           update_grad(new_grad.clone());
         } else {
           if (new_grad.is_mkldnn()) {
@@ -158,8 +191,8 @@ struct TORCH_API AccumulateGrad : public Node {
       } else if (!at::inplaceIsVmapCompatible(variable_grad, new_grad)) {
         // Ideally we'd perform an in-place operation to avoid changing
         // the grad tensor. However, if that's impossible because the grads
-        // are vmap-incompatible (See NOTE: [vmap-incompatible in-place operations]),
-        // then we just add them out-of-place.
+        // are vmap-incompatible (See NOTE: [vmap-incompatible in-place
+        // operations]), then we just add them out-of-place.
         auto result = variable_grad + new_grad;
         CHECK_RESULT(result, variable);
         update_grad(std::move(result));
@@ -170,8 +203,9 @@ struct TORCH_API AccumulateGrad : public Node {
         // 1. `variable_grad` is sparse, and `new_grad` is sparse.
         // 2. `variable_grad` is dense, and `new_grad` is sparse.
         // 3. `variable_grad` is dense, and `new_grad` is dense.
+        // 4. `variable_grad` is mkldnn, and `new_grad` is mkldnn.
         //
-        // In all of these three cases, `variable_grad += new_grad` is a
+        // In all of these four cases, `variable_grad += new_grad` is a
         // valid operation which adds `new_grad` to `variable_grad` in
         // place. `variable_grad` is thus still referring to the same tensor
         // after the operation.
@@ -189,8 +223,9 @@ struct TORCH_API AccumulateGrad : public Node {
         //   variable_grad += new_grad;
         // } else {
         //   result = at::empty_strided(variable.sizes(), variable.strides(),
-        //                              variable.options().memory_format(c10::nullopt));
-        //   update_grad(at::native::add_out(result, variable_grad, new_grad, 1.0);
+        //                              variable.options().memory_format(std::nullopt));
+        //   update_grad(at::native::add_out(result, variable_grad,
+        //   new_grad, 1.0);
         // }
         // However, that accumulation is sometimes in place and sometimes not,
         // which may break user code.
@@ -198,11 +233,13 @@ struct TORCH_API AccumulateGrad : public Node {
     } else {
       at::Tensor result;
       if (variable_grad.is_sparse() && !new_grad.is_sparse()) {
-        // CPU backend throws an error on sparse + dense, so prefer dense + sparse here.
+        // CPU backend throws an error on sparse + dense, so prefer dense +
+        // sparse here.
         result = new_grad + variable_grad;
       } else {
         // Assumes operator+ result typically matches strides of first arg,
-        // and hopes variable_grad was originally created obeying layout contract.
+        // and hopes variable_grad was originally created obeying layout
+        // contract.
         result = variable_grad + new_grad;
       }
       CHECK_RESULT(result, variable);
@@ -216,19 +253,23 @@ struct TORCH_API AccumulateGrad : public Node {
       // such that the stashed grad is likely to have the right strides if
       // either variable_grad or new_grad already has the right strides.
       // We could enforce the contract with certainty by saying
-      // auto result = variable_grad + new_grad (or vice versa), checking result's
-      // layout, and copying to an obedient clone if necessary before update_grad.
-      // The copy would require another gmem pass.  We can't create empty result with
-      // the right layout then add_out into it with a single kernel, because GradMode
-      // is enabled in this branch, and add_out isn't differentiable.
-      // Maybe more trouble than it's worth.
+      // auto result = variable_grad + new_grad (or vice versa), checking
+      // result's layout, and copying to an obedient clone if necessary before
+      // update_grad. The copy would require another gmem pass.  We can't create
+      // empty result with the right layout then add_out into it with a single
+      // kernel, because GradMode is enabled in this branch, and add_out isn't
+      // differentiable. Maybe more trouble than it's worth.
     }
   }
 
+  void compiled_args(CompiledNodeArgs& args) const override;
+  variable_list apply_with_saved(
+      const variable_list& inputs,
+      SwapSavedVariables& saved) override;
+
   Variable variable;
 };
 
 #undef CHECK_RESULT
 
-} // namespace autograd
-} // namespace torch
+} // namespace torch::autograd
