@@ -2,18 +2,46 @@
 
 #include <torch/csrc/jit/jit_log.h>
 
+#include <c10/util/irange.h>
+#include <torch/csrc/jit/api/function_impl.h>
 #include <torch/csrc/jit/ir/alias_analysis.h>
+#include <torch/csrc/jit/passes/autocast.h>
 #include <torch/csrc/jit/passes/clear_profiling.h>
+#include <torch/csrc/jit/passes/eliminate_no_ops.h>
 #include <torch/csrc/jit/passes/inliner.h>
+#include <torch/csrc/jit/passes/lower_tuples.h>
 #include <torch/csrc/jit/runtime/graph_executor_impl.h>
 
 #include <stack>
+#include <utility>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 namespace {
 
+std::vector<std::string> splitName(const std::string& name) {
+  std::vector<std::string> result;
+  std::string sub_name;
+  std::istringstream name_stream(name);
+  while (std::getline(name_stream, sub_name, '.')) {
+    result.push_back(std::move(sub_name));
+  }
+  return result;
+}
+
+template <typename Iter>
+std::string concatName(const Iter& begin, const Iter& end) {
+  std::string combined_name;
+  for (Iter it = begin; it != end; ++it) {
+    const std::string& sub_name = *it;
+    if (!combined_name.empty()) {
+      combined_name += ".";
+    }
+    combined_name += sub_name;
+  }
+  return combined_name;
+}
+
 class AttributePropagator {
  public:
   AttributePropagator(
@@ -24,33 +52,36 @@ class AttributePropagator {
       : module_(module),
         freezeInterfaces_(freezeInterfaces),
         preserveParameters_(preserveParameters) {
-    // Currently only top level attributes and functions can  be preserved
-    // explicitly.
     auto checkName = [this](std::string& name) {
-      if (module_.hasattr(name)) {
-        auto attr = module_.attr(name);
+      const auto resolved_name = resolveName(name);
 
-        // Freezing client wants to presever this submodule. When cleaning
-        // the frozen module, make sure it will be preserved entirely.
-        if (attr.isModule()) {
-          preservedSubModule_.insert(attr.toModule()._ivalue());
+      if (resolved_name) {
+        const auto& parent_module = resolved_name->first;
+        const auto& attr_name = resolved_name->second;
+        if (parent_module.hasattr(attr_name)) {
+          auto value = parent_module.attr(attr_name);
+          // Freezing client wants to preserve this submodule. When cleaning
+          // the frozen module, make sure it will be preserved entirely.
+          if (value.isModule()) {
+            preservedSubModule_.insert(value.toModule()._ivalue());
+          }
+          insertMutableAttr(attr_name, value, parent_module._ivalue());
+        } else {
+          auto fn = parent_module.get_method(attr_name);
+          preservedMethods_.insert(&fn.function());
         }
-        insertMutableAttr(name, attr, module_._ivalue());
         return true;
       }
 
-      for (auto& fn : module_.type()->methods()) {
-        if (fn->name() == name) {
-          preservedMethods_.insert(fn);
-          return true;
-        }
-      }
       return false;
     };
 
-    // forward is preserved by default.
-    auto method = module_.get_method("forward");
-    preservedMethods_.insert(&method.function());
+    // forward is preserved by default, but
+    // not all modules have a forward function defined
+    if (module_.find_method("forward")) {
+      auto method = module_.get_method("forward");
+      preservedMethods_.insert(&method.function());
+    }
 
     for (auto name : preservedAttrs) {
       TORCH_CHECK(checkName(name), "Unknown name: " + name);
@@ -83,17 +114,34 @@ class AttributePropagator {
       ClearProfilingInformation(subgraph);
     };
     auto applyOptimizations = [](std::shared_ptr<Graph>& subgraph) {
+#ifndef C10_MOBILE
+      Autocast(subgraph);
+#endif
       runOptimization(
-          subgraph, /* unroll? */ false, /* const_prop_user_classes? */ false);
+          subgraph,
+          /* unroll_non_constant_loops? */ false,
+          /* const_prop_user_classes? */ false);
+      EliminateNoOps(subgraph);
+      LowerSimpleTuples(subgraph);
     };
 
+    std::unordered_map<std::string, std::unordered_set<std::string>>
+        interfacesToReassignType;
+
     for (auto function : preservedMethods_) {
       GRAPH_DEBUG("Analyzing function: " + function->name());
-      auto graph = function->graph();
+      auto graph = toGraphFunction(*function).graph();
       optimizeSubGraphs(graph, applyInline);
       if (freezeInterfaces_) {
-        inlineInterfaceCalls(graph);
+        inlineInterfaceCalls(graph, interfacesToReassignType);
       }
+    }
+
+    reassignInterfaceTypes(interfacesToReassignType);
+
+    for (auto function : preservedMethods_) {
+      GRAPH_DEBUG("Recording mutable attrs for function: " + function->name());
+      auto graph = toGraphFunction(*function).graph();
       // Record Attributes that are explicitly set in the module.
       // They cannot be folded.
       recordMutableAttrs(graph);
@@ -101,7 +149,7 @@ class AttributePropagator {
 
     for (auto function : preservedMethods_) {
       GRAPH_DEBUG("Propagating function: " + function->name());
-      auto graph = function->graph();
+      auto graph = toGraphFunction(*function).graph();
       propagateAttributes(graph);
       optimizeSubGraphs(graph, applyOptimizations);
     }
@@ -110,6 +158,97 @@ class AttributePropagator {
   }
 
  private:
+  using ResolvedName = std::pair<Module, std::string>;
+
+  // Try to resolve qualified names (submodule1.submodule2.foo). If
+  // the qualified name exists in the root module, return the unqualified
+  // attribute/function name and the parent module. Else, return nullopt.
+  // Examples:
+  // submodule1.submodule2.foo -> {submodule2, "foo"}
+  // submodule1.non_existent_module.foo -> nullopt
+  std::optional<ResolvedName> resolveName(const std::string& name) {
+    auto sub_names = splitName(name);
+    if (sub_names.empty()) {
+      return std::nullopt;
+    }
+    auto& attr_name = sub_names.back();
+    auto cur_module = module_;
+    std::vector<ResolvedName> attr_infos;
+    attr_infos.reserve(sub_names.size() - 1);
+
+    for (size_t i = 0; i < sub_names.size() - 1; ++i) {
+      bool found = false;
+      const auto& sub_name = sub_names[i];
+      for (const auto& child_module : cur_module.named_children()) {
+        if (child_module.name == sub_name) {
+          attr_infos.emplace_back(cur_module._ivalue(), child_module.name);
+          cur_module = child_module.value;
+          found = true;
+          break;
+        }
+      }
+      if (!found) {
+        return std::nullopt;
+      }
+    }
+
+    if (cur_module.hasattr(attr_name) || cur_module.find_method(attr_name)) {
+      // We don't want to mark these modules as mutable yet; that could
+      // interfere with the inlining procedure. Instead, we'll record
+      // the fact that the user wants to preserve them. They will be
+      // processed during clean-up preparation (recordReferenceAttrs)
+      for (auto& attr_info : attr_infos) {
+        const auto& parent_module = attr_info.first;
+        auto& sub_name = attr_info.second;
+        userPreservedAttrs_[parent_module._ivalue()].insert(
+            std::move(sub_name));
+      }
+      return std::make_pair(std::move(cur_module), std::move(attr_name));
+    }
+
+    return std::nullopt;
+  }
+
+  bool _loadModulePath(Value* input, std::shared_ptr<Graph>& graph) {
+    Node* node = input->node();
+    names_.clear();
+    while (!(node->outputs()[0]->type() == graph->inputs()[0]->type())) {
+      if (node->kind() == prim::GetAttr) {
+        names_.push_front(node->s(attr::name));
+        node = node->inputs()[0]->node();
+      } else {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  std::optional<std::deque<std::string>> getModulePath(
+      Value* input,
+      std::shared_ptr<Graph>& graph) {
+    bool success = _loadModulePath(input, graph);
+    if (!success) {
+      return std::nullopt;
+    }
+    return names_;
+  }
+
+  template <typename Iter>
+  bool getModuleFromPath(
+      Module& attrModule,
+      const Iter& begin,
+      const Iter& end) {
+    for (Iter it = begin; it != end; ++it) {
+      const std::string& moduleName = *it;
+      if (preservedAttrs_.count(attrModule.attr(moduleName))) {
+        return false;
+      }
+      attrModule = attrModule.attr(moduleName).toModule();
+    }
+    return true;
+  }
+
   // findConstantAttr function locates the sub Module where attributes are
   // defined. The algorithm chases getAttr chains to locate the submodules.
   // For example:
@@ -160,22 +299,14 @@ class AttributePropagator {
       return false;
     }
 
-    Node* node = input->node();
-    names_.clear();
-    while (!(node->outputs()[0]->type() == graph->inputs()[0]->type())) {
-      if (node->kind() == prim::GetAttr) {
-        names_.push_front(node->s(attr::name));
-        node = node->inputs()[0]->node();
-      } else {
-        return false;
-      }
+    // loads the path into this->names_
+    if (!_loadModulePath(input, graph)) {
+      return false;
     }
 
-    for (auto& moduleName : names_) {
-      if (preservedAttrs_.count(attrModule.attr(moduleName))) {
-        return false;
-      }
-      attrModule = attrModule.attr(moduleName).toModule();
+    // reassigns attrModule to the module in names_
+    if (!getModuleFromPath(attrModule, names_.begin(), names_.end())) {
+      return false;
     }
 
     auto attr = attrModule.attr(name);
@@ -211,7 +342,7 @@ class AttributePropagator {
   void recordMutableAttrs(std::shared_ptr<Graph>& graph) {
     std::stack<Block*> blocks({graph->block()});
     std::unique_ptr<AliasDb> aliasDb =
-        torch::make_unique<AliasDb>(graph, /* isFrozen */ true);
+        std::make_unique<AliasDb>(graph, /* isFrozen */ true);
     while (!blocks.empty()) {
       Block* block = blocks.top();
       blocks.pop();
@@ -220,11 +351,11 @@ class AttributePropagator {
           blocks.push(sub_block);
         }
 
-        // Modules with prim::ModuleDictIndex cannot be frozen because they
+        // Modules with prim::ModuleContainerIndex cannot be frozen because they
         // return InterfaceTypes.
         TORCH_CHECK(
-            n->kind() != prim::ModuleDictIndex,
-            "Freezing modules containing prim::ModuleDictIndex is not supported");
+            n->kind() != prim::ModuleContainerIndex,
+            "Freezing modules containing prim::ModuleContainerIndex is not supported");
 
         if (n->kind() == prim::SetAttr || n->kind() == prim::GetAttr) {
           // By default if interface attributes are present then fail freezing.
@@ -264,6 +395,7 @@ class AttributePropagator {
           applyToForkSubgraph(
               n,
               graph,
+              // NOLINTNEXTLINE(modernize-avoid-bind)
               std::bind(
                   &AttributePropagator::recordMutableAttrs,
                   *this,
@@ -298,32 +430,31 @@ class AttributePropagator {
       }
     } else if (attr.isTuple()) {
       auto tuple = std::move(attr).toTuple();
-      std::vector<IValue>& elems = tuple->elements();
-      for (auto& elem : elems) {
-        elem = overrideGradient(elem);
+      const auto& elems = tuple->elements();
+      for (const auto idx : c10::irange(elems.size())) {
+        tuple->unsafeSetElement(idx, overrideGradient(elems[idx]));
       }
       attr = std::move(tuple);
-
     } else if (attr.isList()) {
       c10::List<IValue> elems = std::move(attr).toList();
-      for (size_t i = 0; i < elems.size(); i++) {
+      for (const auto i : c10::irange(elems.size())) {
         elems.set(i, overrideGradient(elems.extract(i)));
       }
-      attr = std::move(elems);
+      attr = elems;
     } else if (attr.isGenericDict()) {
       auto dict = std::move(attr).toGenericDict();
       for (const auto& pair : dict) {
         auto val = pair.value();
-        val = overrideGradient(val);
+        val = overrideGradient(std::move(val));
       }
-      attr = std::move(dict);
+      attr = dict;
     } else if (attr.isObject() && !attr.toObjectRef().type()->is_module()) {
       auto obj_type = attr.type()->expect<ClassType>();
       auto obj_value = std::move(attr).toObject();
       auto sub_attributes = obj_type->getAttributes();
       for (const auto& sub_attr : sub_attributes) {
         auto sub_attr_val = obj_value->getAttr(sub_attr.getName());
-        sub_attr_val = overrideGradient(sub_attr_val);
+        sub_attr_val = overrideGradient(std::move(sub_attr_val));
       }
       return obj_value;
     }
@@ -342,24 +473,63 @@ class AttributePropagator {
       if (user_node->kind() == prim::CallMethod) {
         const std::string& methodName = user_node->s(attr::name);
         Function& function = class_type->getMethod(methodName);
-        if (!function.isGraphFunction()) {
-          continue;
-        }
-        GRAPH_UPDATE(
-            "Inlining interface method '",
-            function.name(),
-            "' to ",
-            *user_node);
+        if (auto graphFunction = tryToGraphFunction(function)) {
+          GRAPH_UPDATE(
+              "Inlining interface method '",
+              function.name(),
+              "' to ",
+              *user_node);
 
-        GRAPH_UPDATE("Function body: ", *function.optimized_graph());
-        inlineCallTo(user_node, &function);
-        inlined = true;
+          GRAPH_UPDATE("Function body: ", graphFunction->optimized_graph());
+          inlineCallTo(user_node, graphFunction);
+          inlined = true;
+        }
       }
     }
     return inlined;
   }
 
-  void inlineInterfaceCalls(std::shared_ptr<Graph>& graph) {
+  //   [Note: Inlining interfaces strategy]
+  // There's two structures that are relevant to freezing:
+  // - the graph describing the computation in a method
+  // - the module describing the data structure of the module instance.
+  //
+  // First, in inlineInterfaceCalls, we inline interfaces. This is done in a
+  // separate step from normal inlining because CallMethod on an interface type
+  // requires extra steps compared to inlining a normal CallMethod.
+  //
+  // Next we need to simplify the structure of the module data structure, which
+  // is done for the most part by the usual steps in cleanupFrozenModule.
+  //
+  // However, there's a complication that comes from the fact that within a
+  // method, you can change the value of an interface to another module that
+  // implements that interface.
+  //
+  // For example:
+  //
+  // impl: MyInterface
+  // ...
+  // def forward(self, x):
+  //     if x > 0:
+  //         self.impl = my_interface_impl
+  //
+  // This is disallowed in freezing, because in this case we can't flatten out
+  // the module structure, since the type of self.impl will change.
+  //
+  // To handle this, we do the following:
+  //   1. inlineInterfaceCalls:
+  //     a. inline the graph, and in the process record all interfaces
+  //     b. simultaneously, check (throw error) for disallowed SetAttr calls.
+  //   2. call reassignInterfaceTypes, which reassigns interface types to their
+  //      concrete types. This is done in a separate step to avoid interfering
+  //      with inlineInterfaceCalls (note: this may not need to be done as a
+  //      separate step)
+  //   3. eventually cleanupFrozenModule will reorder the module data structure
+  //      and it will expect that all interface types have been removed.
+  void inlineInterfaceCalls(
+      std::shared_ptr<Graph>& graph,
+      std::unordered_map<std::string, std::unordered_set<std::string>>&
+          interfacesToRetype) {
     auto block = graph->block();
     std::stack<Block*> blocks({block});
 
@@ -385,19 +555,79 @@ class AttributePropagator {
           inlineInterfaceCall(n, attr);
           // Reset the GetAttr to concrete module type.
           n->output()->setType(attr.type());
+
+          // Record this so that we can reassign the type later
+          // in reassignInterfaceTypes()
+          // See [Note: Inlining interfaces strategy]
+          auto path = getModulePath(input, graph);
+          TORCH_INTERNAL_ASSERT(path.has_value());
+          auto path_str = concatName(path->begin(), path->end());
+          interfacesToRetype[path_str].insert(name);
+        } else if (n->kind() == prim::SetAttr) {
+          // Check to make sure we're not assigning the value of any parameters
+          // that are interface types.
+          // See [Note: Inlining interfaces strategy]
+          auto name = n->s(attr::name);
+          auto attrModule = module_;
+          auto input = n->inputs()[0];
+
+          if (!input->type()->cast<InterfaceType>() &&
+              !input->type()->expectRef<ClassType>().is_module()) {
+            // we only care if we're setattr["thing"](%mod) if %mod
+            continue;
+          }
+
+          // note: this will modify attrModule until it is the parent of the
+          // "name" attr. In other words, attrModule is now the module that
+          // matches "input".
+          // We can't use findConstantAttr in case the base item is an object,
+          // instead of a module/interface.
+          auto path = getModulePath(input, graph);
+          TORCH_INTERNAL_ASSERT(path.has_value());
+          getModuleFromPath(attrModule, path->begin(), path->end());
+
+          const auto& attrType = attrModule.type()->getAttribute(name);
+          TORCH_INTERNAL_ASSERT(
+              !attrType->cast<InterfaceType>(),
+              "Freezing does not support SetAttr on an interface type. ",
+              "SetAttr is attempted on '",
+              name,
+              "'");
         } else if (n->kind() == prim::fork) {
           applyToForkSubgraph(
               n,
               graph,
+              // NOLINTNEXTLINE(modernize-avoid-bind)
               std::bind(
                   &AttributePropagator::inlineInterfaceCalls,
                   *this,
-                  std::placeholders::_1));
+                  std::placeholders::_1,
+                  interfacesToRetype));
         }
       }
     }
   }
 
+  // See [Note: Inlining interfaces strategy]
+  // This modifies the internal structure of module types to reassign the
+  // type from an interface type to its concrete type.
+  void reassignInterfaceTypes(
+      const std::unordered_map<std::string, std::unordered_set<std::string>>&
+          interfacesToRetype) {
+    for (const auto& it : interfacesToRetype) {
+      const std::string& modulePath = it.first;
+      const std::vector<std::string>& splitPath = splitName(modulePath);
+      Module attrModule = module_;
+      getModuleFromPath(attrModule, splitPath.begin(), splitPath.end());
+
+      for (const std::string& name : it.second) {
+        auto subvalue = attrModule.attr(name);
+        auto subvalueType = subvalue.type();
+        attrModule.type()->unsafeChangeAttributeType(name, subvalueType);
+      }
+    }
+  }
+
   void propagateAttributes(std::shared_ptr<Graph>& graph) {
     std::unordered_map<ModulePtr, std::unordered_map<std::string, Value*>>
         attrValues;
@@ -456,6 +686,16 @@ class AttributePropagator {
             } else {
               attr = overrideGradient(attr);
             }
+            if (attr.isObject()) {
+              if (object_memo_.count(attr.toObject())) {
+                attr = object_memo_[attr.toObject()];
+              } else {
+                auto weak_class_obj =
+                    attr.toObject()->copy_to_weak_compilation_ref();
+                object_memo_[attr.toObject()] = weak_class_obj;
+                attr = weak_class_obj;
+              }
+            }
             if (auto attrVal = tryInsertConstant(*graph, attr)) {
               paramConst = *attrVal;
             } else {
@@ -484,6 +724,7 @@ class AttributePropagator {
           applyToForkSubgraph(
               n,
               graph,
+              // NOLINTNEXTLINE(modernize-avoid-bind)
               std::bind(
                   &AttributePropagator::propagateAttributes,
                   *this,
@@ -522,7 +763,7 @@ class AttributePropagator {
 
   bool moduleEscapes(Module& subModule, std::shared_ptr<Graph>& graph) {
     for (auto& output : graph->outputs()) {
-      if (subModule.type()->isSubtypeOf(output->type())) {
+      if (subModule.type()->isSubtypeOf(*output->type())) {
         return true;
       }
     }
@@ -561,15 +802,16 @@ class AttributePropagator {
   // 3) Remove non public unreferenced methods.
   void cleanupFrozenModule() {
     for (auto function : preservedMethods_) {
-      auto graph = function->graph();
+      auto graph = toGraphFunction(*function).graph();
       recordReferencedAttrs(graph);
       handleSharedClassType(module_, graph);
       removeExtraWaitCalls(graph->block());
+      toGraphFunction(*function).clear_optimized_graphs();
     }
     removeUnusedAttrs();
   }
 
-  // Prepraring for clean up phase. At this point, record all subModules that
+  // Preparing for clean up phase. At this point, record all subModules that
   // contains mutable attributes.
   void recordReferencedAttrs(std::shared_ptr<Graph>& graph) {
     std::stack<Block*> blocks({graph->block()});
@@ -609,6 +851,7 @@ class AttributePropagator {
           applyToForkSubgraph(
               n,
               graph,
+              // NOLINTNEXTLINE(modernize-avoid-bind)
               std::bind(
                   &AttributePropagator::recordReferencedAttrs,
                   *this,
@@ -616,6 +859,16 @@ class AttributePropagator {
         }
       }
     }
+    // We have to process the attributes that the user wants to preserve
+    // separately since it's possible that the user-preserved module is
+    // never referenced in the graph.
+    for (const auto& attr_info : userPreservedAttrs_) {
+      const auto& parent_module = attr_info.first;
+      for (const auto& attr_name : attr_info.second) {
+        const auto value = parent_module->getAttr(attr_name);
+        insertMutableAttr(attr_name, value, parent_module);
+      }
+    }
   }
 
   // This function recursively iterates over submodules to identify
@@ -628,19 +881,19 @@ class AttributePropagator {
     auto type = module.type();
     size_t N = type->numAttributes();
     if (moduleEscapes(module, graph)) {
-      // Perserve all its attributes and methods.
+      // Preserve all its attributes and methods.
       attrsToKeep_[type].insert(N);
       return;
     }
     auto it2 = preservedScalarAttrs_.find(module._ivalue());
     SharedTypeSubModules_[type].insert(module._ivalue());
     attrsToKeep_[type].insert({});
-    for (size_t i = 0; i < N; ++i) {
+    for (const auto i : c10::irange(N)) {
       auto name = type->getAttributeName(i);
       auto attr = module.attr(name);
       auto attrTy = attr.type();
 
-      bool isMutable;
+      bool isMutable = false;
       if (AliasDb::isMutableType(attrTy)) {
         isMutable = preservedAttrs_.count(attr);
       } else {
@@ -650,17 +903,10 @@ class AttributePropagator {
       if (isMutable) {
         attrsToKeep_[type].insert(i);
         if (attr.isModule()) {
-          // FIXME: This error is conservative. Detected an interface module
-          // that cannot be fully inlined away because of side effects.
-          // TODO: We could allow freezing in this case but we would need to
-          // 1) Change the module type to use the concrete type (attrTy).
-          // Probably first unsafe remove attribute and add it using concrete
-          // type.
-          // 2) Fail if there is any setattr to an interface attribute bc
-          // everything is inlined based on old value of this attribute.
+          // See [Note: Inlining interfaces strategy]
           TORCH_CHECK(
               !type->getAttribute(i)->cast<InterfaceType>(),
-              "failed to freeze interface attribute '" + name + "'");
+              "Unexpected interface attribute '" + name + "' during freezing");
 
           auto attrModule = attr.toModule();
           handleSharedClassType(attrModule, graph);
@@ -670,7 +916,7 @@ class AttributePropagator {
   }
 
   // Remove unused attributes and methods for each sub module of the frozen
-  // module. This function iterates over the Calsstypes of its submodule
+  // module. This function iterates over the Classtypes of its submodule
   // attributes including its own type.
   void removeUnusedAttrs() {
     std::vector<std::string> attrsToRemove;
@@ -681,13 +927,13 @@ class AttributePropagator {
       if (it.second.count(N)) {
         continue;
       }
-      for (size_t i = 0; i < N; ++i) {
+      for (const auto i : c10::irange(N)) {
         if (it.second.count(i) == 0) {
           attrsToRemove.push_back(type->getAttributeName(i));
         }
       }
       for (auto& fn : type->methods()) {
-        if (preservedMethods_.count(fn) && *type == *module_.type()) {
+        if (preservedMethods_.count(fn)) {
           continue;
         }
         funcsToRemove.push_back(fn);
@@ -744,7 +990,31 @@ class AttributePropagator {
 
   // Contains the attributes names (e.g. {"self", "subModule", "a"}
   std::deque<std::string> names_;
+
+  // see [Constant Object Weak CompilationUnit Reference]
+  std::unordered_map<
+      c10::intrusive_ptr<at::ivalue::Object>,
+      c10::intrusive_ptr<at::ivalue::Object>>
+      object_memo_;
+
+  // Contains names of attributes that the user wants to preserve with
+  // their owning modules.
+  std::unordered_map<ModulePtr, std::unordered_set<std::string>>
+      userPreservedAttrs_;
+
 }; // class AttributePropagator
+
+void checkModuleDoesNotReturnSelf(const Module& module) {
+  if (module.find_method("forward")) {
+    Method method = module.get_method("forward");
+    // Check that module does not return itself.
+    for (auto& output : method.graph()->outputs()) {
+      TORCH_CHECK(
+          output->type() != module.type(),
+          "attempted to freeze a module that return itself");
+    }
+  }
+}
 } // namespace
 
 Module freeze_module(
@@ -752,13 +1022,7 @@ Module freeze_module(
     std::vector<std::string> preservedAttrs,
     bool freezeInterfaces,
     bool preserveParameters) {
-  Method method = module.get_method("forward");
-  // Check that module does not return itself.
-  for (auto& output : method.graph()->outputs()) {
-    TORCH_CHECK(
-        output->type() != module.type(),
-        "attempted to freeze a module that return itself");
-  }
+  checkModuleDoesNotReturnSelf(module);
 
   auto moduleClone = module.clone(true);
   AttributePropagator attrPropagator(
@@ -767,5 +1031,16 @@ Module freeze_module(
   return moduleClone;
 }
 
-} // namespace jit
-} // namespace torch
+void freeze_module_inplace(
+    Module* module,
+    std::vector<std::string> preservedAttrs,
+    bool freezeInterfaces,
+    bool preserveParameters) {
+  TORCH_CHECK(module != nullptr, "module cannot be nullptr");
+  checkModuleDoesNotReturnSelf(*module);
+  AttributePropagator attrPropagator(
+      *module, preservedAttrs, freezeInterfaces, preserveParameters);
+  attrPropagator.run();
+}
+
+} // namespace torch::jit
