@@ -1,11 +1,16 @@
 #include <torch/csrc/jit/python/python_ir.h>
 
+#include <ATen/core/jit_type.h>
 #include <pybind11/pybind11.h>
+#include <torch/csrc/Device.h>
+#include <torch/csrc/Dtype.h>
+#include <torch/csrc/api/include/torch/python.h>
 #include <torch/csrc/jit/ir/alias_analysis.h>
 #include <torch/csrc/jit/ir/ir.h>
 #include <torch/csrc/jit/passes/canonicalize.h>
 #include <torch/csrc/jit/passes/onnx/helper.h>
 #include <torch/csrc/jit/passes/shape_analysis.h>
+#include <torch/csrc/jit/passes/symbolic_shape_analysis.h>
 #include <torch/csrc/jit/python/pybind.h>
 #include <torch/csrc/jit/python/python_tracer.h>
 #include <torch/csrc/jit/runtime/argument_spec.h>
@@ -18,8 +23,10 @@
 #include <sstream>
 #include <utility>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
+
+// Controls whether graph source ranges are printed by default
+bool global_print_source_ranges = true;
 
 Symbol ConcretePythonOp::Kind = prim::PythonOp;
 
@@ -74,33 +81,6 @@ std::ostream& printPyObject(std::ostream& out, const THPObjectPtr& obj) {
   }
 }
 
-std::vector<Node*> findAllNodes(
-    c10::ArrayRef<torch::jit::Block*> blocks,
-    Symbol kind,
-    bool recurse = true) {
-  std::vector<Node*> ret;
-  for (Block* block : blocks) {
-    for (Node* n : block->nodes()) {
-      if (n->kind() == kind) {
-        ret.push_back(n);
-      }
-      if (recurse) {
-        auto nodes = findAllNodes(n->blocks(), kind, recurse);
-        ret.insert(ret.end(), nodes.begin(), nodes.end());
-      }
-    }
-  }
-  return ret;
-}
-
-std::vector<Node*> findAllNodes(
-    Block* block,
-    Symbol kind,
-    bool recurse = true) {
-  std::vector<Block*> blocks = {block};
-  return findAllNodes(blocks, kind, recurse);
-}
-
 Node* findNode(
     c10::ArrayRef<torch::jit::Block*> blocks,
     Symbol kind,
@@ -136,6 +116,7 @@ std::string ConcretePythonOp::name() const {
 }
 
 void ConcretePythonOp::cloneFrom(Node* other_) {
+  // NOLINTNEXTLINE(bugprone-parent-virtual-call)
   Node::cloneFrom(other_);
   auto other = other_->cast<ConcretePythonOp>();
   this->cconv = other->cconv;
@@ -150,24 +131,24 @@ void ConcretePythonOp::cloneFrom(Node* other_) {
 // recover the autograd.Function instance, if this PythonOp's function
 // was originally SomeFunction.apply
 // used in ONNX for discovering symbolics
-c10::optional<THPObjectPtr> ConcretePythonOp::autogradFunction() const {
+std::optional<THPObjectPtr> ConcretePythonOp::autogradFunction() const {
   pybind11::gil_scoped_acquire gil;
   // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
   py::handle obj = const_cast<PyObject*>(pyobj.get());
 
   auto r = py::getattr(obj, "__self__", py::none());
   if (r.is_none())
-    return c10::nullopt;
+    return std::nullopt;
 
   auto apply = py::getattr(r, "apply", py::none());
   if (apply.is_none())
-    return c10::nullopt;
+    return std::nullopt;
 
   auto c = PyObject_RichCompareBool(apply.ptr(), obj.ptr(), Py_NE);
   if (PyErr_Occurred())
     throw py::error_already_set();
   if (c)
-    return c10::nullopt;
+    return std::nullopt;
 
   return THPObjectPtr(r.release().ptr());
 }
@@ -209,11 +190,52 @@ Node* Graph::createPythonOp(
 
 void initPythonIRBindings(PyObject* module_) {
   auto m = py::handle(module_).cast<py::module>();
+
+  py::class_<AliasDb, std::shared_ptr<AliasDb>>(m, "AliasDb")
+      .def("dump", &AliasDb::dump)
+      .def("to_graphviz_str", &AliasDb::toGraphviz)
+      .def(
+          "may_contain_alias",
+          [&](AliasDb& db, Value* v1, Value* v2) {
+            return db.mayContainAlias(v1, v2);
+          })
+      .def(
+          "has_writers",
+          [&](AliasDb& db, Value* v1) { return db.hasWriters(v1); })
+      .def("__str__", &AliasDb::toString)
+      .def(
+          "move_after_topologically_valid",
+          [](AliasDb& db, Node* n, Node* movePoint) {
+            return db.moveAfterTopologicallyValid(n, movePoint);
+          })
+      .def(
+          "move_before_topologically_valid",
+          [](AliasDb& db, Node* n, Node* movePoint) {
+            return db.moveBeforeTopologicallyValid(n, movePoint);
+          });
 #define GS(name) def(#name, &Graph ::name)
   py::class_<Graph, std::shared_ptr<Graph>>(m, "Graph")
       .def(py::init<>())
-      .def("__repr__", [](Graph& g) { return g.toString(); })
+      .def(
+          "__repr__",
+          [&](Graph& g) { return g.toString(global_print_source_ranges); })
       .def("str", &Graph::toString, py::arg("print_source_ranges") = true)
+      .def_readonly_static(
+          "global_print_source_ranges", &global_print_source_ranges)
+      .def_static(
+          "set_global_print_source_ranges",
+          [&](const bool enabled) { global_print_source_ranges = enabled; },
+          py::arg("enabled") = true)
+      .def(
+          "alias_db",
+          [](std::shared_ptr<Graph> g,
+             bool isFrozen = false,
+             bool descend_function_calls = false) {
+            return std::make_shared<AliasDb>(
+                std::move(g), isFrozen, descend_function_calls);
+          },
+          py::arg("isFrozen") = false,
+          py::arg("descend_function_calls") = false)
       .def(
           "dump_alias_db",
           [](std::shared_ptr<Graph> g) {
@@ -234,26 +256,29 @@ void initPythonIRBindings(PyObject* module_) {
              bool keep_initializers_as_inputs,
              const std::map<std::string, int>& custom_opsets,
              bool add_node_names,
-             bool use_external_data_format,
-             const std::string& onnx_file_path) {
+             const std::string& onnx_file_path,
+             const NodeAttrNameMap& node_attr_to_name) {
             std::string graph;
-            std::shared_ptr<::ONNX_NAMESPACE::ModelProto> model_proto;
-            RawDataExportMap export_map;
-            SymbolDimMap symbol_map;
-            std::tie(model_proto, export_map, symbol_map) = export_onnx(
-                g,
-                initializers,
-                onnx_opset_version,
-                dynamic_axes,
-                defer_weight_export,
-                operator_export_type,
-                strip_doc_string,
-                keep_initializers_as_inputs,
-                custom_opsets,
-                add_node_names,
-                use_external_data_format,
-                onnx_file_path);
-            graph = serialize_model_proto_to_string(model_proto);
+            auto
+                [model_proto,
+                 export_map,
+                 symbol_map,
+                 val_use_external_data_format,
+                 onnx_node_names] =
+                    export_onnx(
+                        g,
+                        initializers,
+                        onnx_opset_version,
+                        dynamic_axes,
+                        defer_weight_export,
+                        operator_export_type,
+                        strip_doc_string,
+                        keep_initializers_as_inputs,
+                        custom_opsets,
+                        add_node_names,
+                        false,
+                        onnx_file_path,
+                        node_attr_to_name);
             std::unordered_map<std::string, py::bytes>
                 python_serialized_export_map;
             for (auto& kv : export_map) {
@@ -267,7 +292,10 @@ void initPythonIRBindings(PyObject* module_) {
             }
             graph = serialize_model_proto_to_string(model_proto);
             return std::make_tuple(
-                py::bytes(graph), python_serialized_export_map);
+                py::bytes(graph),
+                python_serialized_export_map,
+                val_use_external_data_format,
+                onnx_node_names);
           },
           py::arg("initializers"),
           py::arg("onnx_opset_version") = 0,
@@ -279,8 +307,8 @@ void initPythonIRBindings(PyObject* module_) {
           py::arg("keep_initializers_as_inputs") = true,
           py::arg("custom_opsets"),
           py::arg("add_node_names") = true,
-          py::arg("use_external_data_format") = false,
-          py::arg("onnx_file_path") = std::string())
+          py::arg("onnx_file_path") = std::string(),
+          py::arg("node_attr_to_name") = NodeAttrNameMap())
       .def(
           "_pretty_print_onnx",
           [](const std::shared_ptr<Graph>& g,
@@ -316,18 +344,22 @@ void initPythonIRBindings(PyObject* module_) {
           "inputs",
           [](Graph& g) {
             return py::make_iterator(g.inputs().begin(), g.inputs().end());
-          })
+          },
+          py::keep_alive<0, 1>())
       .def(
           "outputs",
           [](Graph& g) {
             return py::make_iterator(g.outputs().begin(), g.outputs().end());
-          })
-      // TODO: Iterator invalidation might make this hazardous
+          },
+          py::keep_alive<0, 1>())
+      // We keep the graph alive while the iterator lives. Destroying
+      // nodes might still be hazardous.
       .def(
           "nodes",
           [](Graph& g) {
             return py::make_iterator(g.nodes().begin(), g.nodes().end());
-          })
+          },
+          py::keep_alive<0, 1>())
       .def(
           "findNode",
           [](Graph& g, const std::string& kind, bool recurse) {
@@ -339,16 +371,25 @@ void initPythonIRBindings(PyObject* module_) {
       .def(
           "findAllNodes",
           [](Graph& g, const std::string& kind, bool recurse) {
-            return findAllNodes(
-                g.block(), Symbol::fromQualString(kind), recurse);
+            return findAllNodes(g, Symbol::fromQualString(kind), recurse);
           },
           "Find all nodes",
           py::arg("kind"),
           py::arg("recurse") = true)
-      .def("addInput", [](Graph& g) { return g.addInput(); })
+      .def(
+          "addInput",
+          [](Graph& g, const std::string& name) { return g.addInput(name); },
+          "Add input to graph with optional name seed",
+          py::arg("name") = "")
       .def("copy", [](Graph& g) { return g.copy(); })
       .GS(eraseInput)
+      .GS(eraseOutput)
       .GS(registerOutput)
+      .def(
+          "permuteInputs",
+          [](Graph& g, const std::vector<size_t>& new_inputs) {
+            g.block()->permuteInputs(new_inputs);
+          })
       .def(
           "create",
           [](Graph& g, const char* str) {
@@ -362,6 +403,12 @@ void initPythonIRBindings(PyObject* module_) {
       .def(
           "create",
           [](Graph& g, const char* str, const std::vector<Value*>& inputs) {
+            TORCH_CHECK_VALUE(
+                std::all_of(
+                    inputs.begin(),
+                    inputs.end(),
+                    [](Value* v) { return (v != nullptr); }),
+                "cannot pass None in inputs");
             return g.create(Symbol::fromQualString(str), inputs);
           })
       .def(
@@ -370,6 +417,12 @@ void initPythonIRBindings(PyObject* module_) {
              const char* str,
              const std::vector<Value*>& inputs,
              size_t noutputs) {
+            TORCH_CHECK_VALUE(
+                std::all_of(
+                    inputs.begin(),
+                    inputs.end(),
+                    [](Value* v) { return (v != nullptr); }),
+                "cannot pass None in inputs");
             return g.create(Symbol::fromQualString(str), inputs, noutputs);
           })
       .def("param_node", [](Graph& g) { return g.block()->param_node(); })
@@ -388,10 +441,62 @@ void initPythonIRBindings(PyObject* module_) {
           })
       .GS(appendNode)
       .GS(prependNode)
+      // NB: insert_point_guard defined over direct modification of insert point
+      .def(
+          "insert_point_guard",
+          [](Graph& g, Node* n) {
+            return py::module::import("torch.jit._ir_utils")
+                .attr("insert_point_guard")(g, n);
+          })
+      .def(
+          "insert_point_guard",
+          [](Graph& g, Block* b) {
+            return py::module::import("torch.jit._ir_utils")
+                .attr("insert_point_guard")(g, b);
+          })
+      .GS(insertPoint)
+      .def("setInsertPoint", [](Graph& g, Node* n) { g.setInsertPoint(n); })
+      .def("setInsertPoint", [](Graph& g, Block* n) { g.setInsertPoint(n); })
+      .def(
+          "insertGraph",
+          [](Graph& g, Graph& callee, const std::vector<Value*>& inputs) {
+            return insertGraph(g, callee, inputs);
+          })
+      .def(
+          "insertGraph",
+          [](Graph& g,
+             Graph& callee,
+             const std::vector<Value*>& inputs,
+             std::unordered_map<Value*, Value*> value_map) {
+            return insertGraph(g, callee, inputs, value_map);
+          })
+      .def(
+          "insert",
+          [](Graph& g, Symbol opname, const std::vector<Value*>& args) {
+            std::vector<NamedValue> args_named;
+            args_named.reserve(args.size());
+            for (Value* v : args) {
+              args_named.emplace_back(v);
+            }
+            return g.insert(opname, args_named);
+          })
+      .def(
+          "makeMultiOutputIntoTuple",
+          [](Graph& g) {
+            auto tup = g.createTuple(g.outputs());
+            tup->insertBefore(g.return_node());
+            for (int64_t i = static_cast<int64_t>(g.outputs().size()) - 1;
+                 i >= 0;
+                 i--) {
+              g.eraseOutput(0);
+            }
+            g.registerOutput(tup->output());
+          })
       .def(
           "insertConstant",
           [](Graph& g, const IValue& ival) { return g.insertConstant(ival); })
       .GS(lint)
+      .def("block", [](Graph& g) { return g.block(); })
       .GS(insertNode);
 #undef GS
 
@@ -434,7 +539,9 @@ void initPythonIRBindings(PyObject* module_) {
       .VS(requires_grad)
       .def(
           "requiresGrad",
-          [](Value& n) { n.type()->expectRef<TensorType>().requiresGrad(); })
+          [](Value& n) {
+            return n.type()->expectRef<TensorType>().requiresGrad();
+          })
       .def("toIValue", [](Value& n) { return toIValue(&n); })
       .def("type", [](Value& v) { return v.type(); });
 #undef VS
@@ -456,7 +563,7 @@ void initPythonIRBindings(PyObject* module_) {
       .def(
           "findAllNodes",
           [](Block& b, const std::string& kind, bool recurse) {
-            return findAllNodes(&b, Symbol::fromQualString(kind), recurse);
+            return findAllNodes(b, Symbol::fromQualString(kind), recurse);
           },
           "Find all nodes",
           py::arg("kind"),
@@ -473,6 +580,7 @@ void initPythonIRBindings(PyObject* module_) {
           })
       .def("returnNode", [](Block& b) { return b.return_node(); })
       .def("paramNode", [](Block& b) { return b.param_node(); })
+      .def("owningNode", [](Block& b) { return b.owningNode(); })
       .def(
           "addNode",
           [](Block& b, const char* str, const std::vector<Value*>& inputs) {
@@ -497,6 +605,9 @@ void initPythonIRBindings(PyObject* module_) {
       .def("inputsSize", [](Node& n) { return n.inputs().size(); })
       .def("outputsSize", [](Node& n) { return n.outputs().size(); })
       .NS(kind)
+      .def("prev", [](Node& n) { return n.prev(); })
+      .def("matches", [](Node& n, const char* s) { return n.matches(s); })
+      .def("owningBlock", [](Node& n) { return n.owningBlock(); })
       .def("inputsAt", [](Node& n, size_t i) { return n.inputs().at(i); })
       .def(
           "inputs",
@@ -507,7 +618,7 @@ void initPythonIRBindings(PyObject* module_) {
           "schema",
           [](Node& n) {
             std::stringstream ss;
-            if (auto sch = n.maybeSchema()) {
+            if (n.maybeSchema()) {
               ss << n.schema();
             } else {
               ss << "(no schema)";
@@ -539,7 +650,16 @@ void initPythonIRBindings(PyObject* module_) {
           py::arg("recurse") = true)
       .def("input", [](Node& n) { return n.input(); })
       .def("output", [](Node& n) { return n.output(); })
+      .def(
+          "getModuleHierarchy",
+          [](Node& n) { return torch::jit::utils::getNodesModuleHierarchy(n); })
+      .def(
+          "namedInput",
+          [](Node& n, const std::string& unqualName) {
+            return n.namedInput(unqualName);
+          })
       .NS(addInput)
+      .NS(copyMetadata)
       .NS(replaceInput)
       .NS(replaceInputWith)
       .NS(replaceAllUsesWith)
@@ -587,12 +707,14 @@ void initPythonIRBindings(PyObject* module_) {
   })
       .CREATE_ACCESSOR(Float, f)
       .CREATE_ACCESSOR(Floats, fs)
+      .CREATE_ACCESSOR(Complex, c)
       .CREATE_ACCESSOR(String, s)
       .CREATE_ACCESSOR(Strings, ss)
       .CREATE_ACCESSOR(Int, i)
       .CREATE_ACCESSOR(Ints, is)
       .CREATE_ACCESSOR(Graph, g)
       .CREATE_ACCESSOR(Graphs, gs)
+      .CREATE_ACCESSOR(IValue, ival)
 #undef CREATE_ACCESSOR
       // Tensor (t_) -- manually written to unwrap the variable into a tensor.
       .def(
@@ -634,18 +756,32 @@ void initPythonIRBindings(PyObject* module_) {
           [](Node& n, const char* name, const at::Tensor& v) {
             return n.t_(
                 Symbol::attr(name),
-                autograd::Variable(v.view(std::vector<int64_t>{}))
-                    .set_requires_grad(false));
+                v.view(std::vector<int64_t>{}).set_requires_grad(false));
           })
       .def(
           "z",
           [](Node& n, const char* name) { return n.t(Symbol::attr(name)); })
+      .def(
+          "ty_",
+          [](Node& n, const char* name, const TypePtr& type) {
+            return n.ty_(Symbol::attr(name), type);
+          })
+      .def(
+          "ty",
+          [](Node& n, const char* name) { return n.ty(Symbol::attr(name)); })
+      .def(
+          "tys_",
+          [](Node& n, const char* name, const std::vector<TypePtr>& types) {
+            return n.tys_(Symbol::attr(name), types);
+          })
+      .def(
+          "tys",
+          [](Node& n, const char* name) { return n.tys(Symbol::attr(name)); })
       .def(
           "zs_",
           [](Node& n, const char* name, TensorsAttr::ValueType v) {
             for (auto& i : v) {
-              i = autograd::Variable(i.view(std::vector<int64_t>{}))
-                      .set_requires_grad(false);
+              i = i.view(std::vector<int64_t>{}).set_requires_grad(false);
             }
             return n.ts_(Symbol::attr(name), std::move(v));
           })
@@ -673,7 +809,7 @@ void initPythonIRBindings(PyObject* module_) {
       });
 
   using ::c10::Type;
-  py::class_<Type, std::shared_ptr<Type>>(m, "Type")
+  py::class_<Type, TypePtr>(m, "Type")
       .def("__repr__", [](Type& t) { return t.annotation_str(); })
       .def(
           "str",
@@ -682,19 +818,21 @@ void initPythonIRBindings(PyObject* module_) {
             s << t;
             return s.str();
           })
+      .def(
+          "containedTypes",
+          [](Type& self) { return self.containedTypes().vec(); })
       .def("kind", [](const Type& t) { return typeKindToString(t.kind()); })
       .def(
           "dim",
           [](Type& t) {
-            auto vshape = t.shared_from_this()->expectRef<TensorType>().sizes();
+            auto vshape = t.expectRef<TensorType>().sizes();
             return vshape.size() ? py::cast(*vshape.size())
                                  : py::cast<py::none>(Py_None);
           })
       .def(
           "undefined",
           [](Type& t) {
-            auto undef =
-                t.shared_from_this()->expectRef<TensorType>().undefined();
+            auto undef = t.expectRef<TensorType>().undefined();
             return undef.has_value() ? py::cast(*undef)
                                      : py::cast<py::none>(Py_None);
           })
@@ -708,6 +846,36 @@ void initPythonIRBindings(PyObject* module_) {
             }
             return py::none();
           })
+      .def(
+          "symbolic_sizes",
+          [](Type& t) -> py::object {
+            if (auto ptt = t.expect<TensorType>()) {
+              auto ss = ptt->symbolic_sizes();
+              if (!ss.rank().has_value()) {
+                return py::none();
+              }
+
+              std::vector<int64_t> ss_vals;
+              for (size_t i = 0; i < *ss.rank(); ++i) {
+                ss_vals.push_back(ss.at(i).value());
+              }
+              return py::cast(ss_vals);
+            }
+            return py::none();
+          })
+      .def(
+          "with_sizes",
+          [](Type& t, std::optional<std::vector<std::optional<int64_t>>> sizes)
+              -> py::object {
+            auto ptt = t.expect<TensorType>();
+            if (!ptt) {
+              return py::none();
+            }
+            if (!sizes) {
+              return py::cast(ptt->withSymbolicShapes(c10::SymbolicShape()));
+            }
+            return py::cast(ptt->withSymbolicShapes(*sizes));
+          })
       .def(
           "varyingSizes",
           [](Type& t) -> py::object {
@@ -737,13 +905,57 @@ void initPythonIRBindings(PyObject* module_) {
       .def(
           "scalarType",
           [](Type& t) {
-            auto scalar_type =
-                t.shared_from_this()->expectRef<TensorType>().scalarType();
+            auto scalar_type = t.expectRef<TensorType>().scalarType();
             return (scalar_type) ? toString(*scalar_type) : nullptr;
           })
+      .def(
+          "device",
+          [](Type& t) -> py::object {
+            auto device = t.expectRef<TensorType>().device();
+            if (!device) {
+              return py::none();
+            }
+            PyObject* thp_device = THPDevice_New(device.value());
+            return py::reinterpret_borrow<py::object>(thp_device);
+            // return toPyObject(device.value());
+          })
+      .def(
+          "with_device",
+          [](Type& t, py::object device) -> py::object {
+            at::Device c_device =
+                python::detail::py_object_to_device(std::move(device));
+            if (auto ptt = t.expect<TensorType>()) {
+              return py::cast(ptt->withDevice(c_device));
+            }
+            return py::none();
+          })
+      .def(
+          "dtype",
+          [](Type& t) -> py::object {
+            auto scalar_type = t.expectRef<TensorType>().scalarType();
+            if (!scalar_type) {
+              return py::none();
+            }
+            THPDtype* thp_dtype = torch::getTHPDtype(*scalar_type);
+            py::object dtype =
+                py::reinterpret_borrow<py::object>((PyObject*)thp_dtype);
+            return dtype;
+          })
+      .def(
+          "with_dtype",
+          [](Type& t, py::object dtype) -> py::object {
+            at::ScalarType scalar_type =
+                python::detail::py_object_to_dtype(std::move(dtype));
+
+            if (auto ptt = t.expect<TensorType>()) {
+              // auto scalar_type = dtype->scalar_type;
+              return py::cast(ptt->withScalarType(scalar_type));
+            }
+            return py::none();
+          })
       .def(
           "__eq__",
-          [](std::shared_ptr<Type>& self, std::shared_ptr<Type>& other) {
+          [](const TypePtr& self, const TypePtr& other) {
             if (!other) {
               return false;
             }
@@ -751,46 +963,63 @@ void initPythonIRBindings(PyObject* module_) {
           })
       .def(
           "isSubtypeOf",
-          [](std::shared_ptr<Type>& self, std::shared_ptr<Type>& other) {
+          [](const TypePtr& self, const TypePtr& other) {
             if (!other) {
               return false;
             }
             return self->isSubtypeOf(other);
           })
-      .def("is_interface_type", [](const std::shared_ptr<Type>& self) {
-        return self->cast<InterfaceType>() != nullptr;
-      });
+      .def(
+          "is_interface_type",
+          [](const TypePtr& self) {
+            return self->castRaw<InterfaceType>() != nullptr;
+          })
+      .def(
+          "requires_grad",
+          [](const TypePtr& self) -> bool { return self->requires_grad(); })
+      .def_property_readonly(
+          "annotation_str", [](const std::shared_ptr<Type>& self) {
+            return self->annotation_str();
+          });
 
-  py::class_<AnyType, Type, std::shared_ptr<AnyType>>(m, "AnyType")
+  py::class_<AnyType, Type, AnyTypePtr>(m, "AnyType")
       .def_static("get", &AnyType::get);
-  py::class_<NumberType, Type, std::shared_ptr<NumberType>>(m, "NumberType")
+  py::class_<NumberType, Type, NumberTypePtr>(m, "NumberType")
       .def_static("get", &NumberType::get);
-  py::class_<IntType, Type, std::shared_ptr<IntType>>(m, "IntType")
+  py::class_<IntType, Type, IntTypePtr>(m, "IntType")
       .def_static("get", &IntType::get);
-  py::class_<FloatType, Type, std::shared_ptr<FloatType>>(m, "FloatType")
+  py::class_<SymIntType, Type, SymIntTypePtr>(m, "SymIntType")
+      .def_static("get", &SymIntType::get);
+  py::class_<SymBoolType, Type, SymBoolTypePtr>(m, "SymBoolType")
+      .def_static("get", &SymBoolType::get);
+  py::class_<FloatType, Type, FloatTypePtr>(m, "FloatType")
       .def_static("get", &FloatType::get);
-  py::class_<ComplexType, Type, std::shared_ptr<ComplexType>>(m, "ComplexType")
+  py::class_<ComplexType, Type, ComplexTypePtr>(m, "ComplexType")
       .def_static("get", &ComplexType::get);
-  py::class_<TensorType, Type, std::shared_ptr<TensorType>>(m, "TensorType")
+  py::class_<TensorType, Type, TensorTypePtr>(m, "TensorType")
       .def_static("get", &TensorType::get)
-      .def_static("getInferred", &TensorType::getInferred);
-  py::class_<BoolType, Type, std::shared_ptr<BoolType>>(m, "BoolType")
+      .def_static("getInferred", &TensorType::getInferred)
+      .def_static("create_from_tensor", [](const at::Tensor& t) {
+        return TensorType::create(t);
+      });
+  py::class_<BoolType, Type, BoolTypePtr>(m, "BoolType")
       .def_static("get", &BoolType::get);
-  py::class_<StringType, Type, std::shared_ptr<StringType>>(m, "StringType")
+  py::class_<StringType, Type, StringTypePtr>(m, "StringType")
       .def_static("get", &StringType::get);
-  py::class_<DeviceObjType, Type, std::shared_ptr<DeviceObjType>>(
-      m, "DeviceObjType")
+  py::class_<DeviceObjType, Type, DeviceObjTypePtr>(m, "DeviceObjType")
       .def_static("get", &DeviceObjType::get);
-  py::class_<StreamObjType, Type, std::shared_ptr<StreamObjType>>(
-      m, "StreamObjType")
+  // TODO(antoniojkim): Add GeneratorType to the public API once its been added
+  //                    to the public documentation
+  py::class_<GeneratorType, Type, GeneratorTypePtr>(m, "_GeneratorType")
+      .def_static("get", &GeneratorType::get);
+  py::class_<StreamObjType, Type, StreamObjTypePtr>(m, "StreamObjType")
       .def_static("get", &StreamObjType::get);
-  py::class_<PyObjectType, Type, std::shared_ptr<PyObjectType>>(
-      m, "PyObjectType")
+  py::class_<PyObjectType, Type, PyObjectTypePtr>(m, "PyObjectType")
       .def_static("get", &PyObjectType::get);
-  py::class_<NoneType, Type, std::shared_ptr<NoneType>>(m, "NoneType")
+  py::class_<NoneType, Type, NoneTypePtr>(m, "NoneType")
       .def_static("get", &NoneType::get);
 
-  py::class_<TupleType, Type, std::shared_ptr<TupleType>>(m, "TupleType")
+  py::class_<TupleType, Type, TupleTypePtr>(m, "TupleType")
       .def(py::init([](std::vector<TypePtr> a) {
         return TupleType::create(std::move(a));
       }))
@@ -801,40 +1030,56 @@ void initPythonIRBindings(PyObject* module_) {
         }
         return types;
       });
-  py::class_<ListType, Type, std::shared_ptr<ListType>>(m, "ListType")
-      .def(py::init([](TypePtr a) { return ListType::create(a); }))
+  py::class_<UnionType, Type, UnionTypePtr>(m, "UnionType")
+      .def(py::init(
+          [](const std::vector<TypePtr>& a) { return UnionType::create(a); }));
+  py::class_<ListType, Type, ListTypePtr>(m, "ListType")
+      .def(py::init([](const TypePtr& a) { return ListType::create(a); }))
       .def_static("ofInts", &ListType::ofInts)
       .def_static("ofTensors", &ListType::ofTensors)
       .def_static("ofFloats", &ListType::ofFloats)
       .def_static("ofComplexDoubles", &ListType::ofComplexDoubles)
       .def_static("ofBools", &ListType::ofBools)
+      .def_static("ofStrings", &ListType::ofStrings)
       .def("getElementType", &ListType::getElementType);
-  py::class_<DictType, Type, std::shared_ptr<DictType>>(m, "DictType")
+  py::class_<DictType, Type, DictTypePtr>(m, "DictType")
       .def(py::init([](TypePtr key, TypePtr value) {
         return DictType::create(std::move(key), std::move(value));
       }))
       .def("getKeyType", &DictType::getKeyType)
       .def("getValueType", &DictType::getValueType);
-  py::class_<OptionalType, Type, std::shared_ptr<OptionalType>>(
-      m, "OptionalType")
-      .def(py::init(
-          [](TypePtr a) { return OptionalType::create(std::move(a)); }))
+  py::class_<OptionalType, Type, OptionalTypePtr>(m, "OptionalType")
+      .def(py::init([](const TypePtr& a) { return OptionalType::create(a); }))
       .def_static("ofTensor", &OptionalType::ofTensor)
       .def("getElementType", &OptionalType::getElementType);
-  py::class_<RRefType, Type, std::shared_ptr<RRefType>>(m, "RRefType")
+  py::class_<RRefType, Type, RRefTypePtr>(m, "RRefType")
       .def(py::init([](TypePtr a) { return RRefType::create(std::move(a)); }))
       .def("getElementType", &RRefType::getElementType);
 
-  py::class_<FutureType, Type, std::shared_ptr<FutureType>>(m, "FutureType")
+  py::class_<FutureType, Type, FutureTypePtr>(m, "FutureType")
       .def(py::init([](TypePtr a) { return FutureType::create(std::move(a)); }))
       .def("getElementType", &FutureType::getElementType);
 
-  py::class_<ClassType, Type, std::shared_ptr<ClassType>>(m, "ClassType")
+  py::class_<AwaitType, Type, AwaitTypePtr>(m, "AwaitType")
+      .def(py::init([](TypePtr a) { return AwaitType::create(std::move(a)); }))
+      .def("getElementType", &AwaitType::getElementType);
+
+  py::class_<ClassType, Type, ClassTypePtr>(m, "ClassType")
       .def(py::init([](const std::string& qualified_name) {
         return get_python_cu()->get_class(c10::QualifiedName(qualified_name));
       }))
-      .def("name", [](ClassType& self) { return self.name()->name(); });
-  py::class_<EnumType, Type, std::shared_ptr<EnumType>>(m, "EnumType")
+      .def("name", [](ClassType& self) { return self.name()->name(); })
+      .def(
+          "qualified_name",
+          [](ClassType& self) { return self.name()->qualifiedName(); })
+      .def("method_names", [](ClassType& self) {
+        std::vector<std::string> method_names;
+        for (const auto* method : self.methods()) {
+          method_names.push_back(method->name());
+        }
+        return method_names;
+      });
+  py::class_<EnumType, Type, EnumTypePtr>(m, "EnumType")
       .def(py::init([](const std::string& qualified_name,
                        TypePtr value_type,
                        const std::vector<py::object>& enum_names_values) {
@@ -843,7 +1088,7 @@ void initPythonIRBindings(PyObject* module_) {
         for (const auto& enum_name_value : enum_names_values) {
           auto enum_name = py::cast<std::string>(enum_name_value.attr("name"));
           auto enum_value = toIValue(enum_name_value.attr("value"), value_type);
-          names_values.emplace_back(std::make_pair(enum_name, enum_value));
+          names_values.emplace_back(enum_name, enum_value);
         }
         return EnumType::create(
             c10::QualifiedName(qualified_name),
@@ -851,8 +1096,7 @@ void initPythonIRBindings(PyObject* module_) {
             std::move(names_values),
             get_python_cu());
       }));
-  py::class_<InterfaceType, Type, std::shared_ptr<InterfaceType>>(
-      m, "InterfaceType")
+  py::class_<InterfaceType, Type, InterfaceTypePtr>(m, "InterfaceType")
       .def(py::init([](const std::string& qualified_name) {
         return get_python_cu()->get_interface(
             c10::QualifiedName(qualified_name));
@@ -898,6 +1142,18 @@ void initPythonIRBindings(PyObject* module_) {
       .def("isAfter", [](Use& self, Use& other_use) {
         return isBeforeOrAfter(self, other_use, false);
       });
+
+  py::class_<torch::jit::ShapeComputeGraphMapping>(
+      m, "_ShapeComputeGraphMapping")
+      .def(
+          "partial_eval_shape_graph",
+          [](ShapeComputeGraphMapping& g) {
+            return g.partial_eval_shape_graph;
+          })
+      .def(
+          "graph_output_to_symbolic_shape_dim",
+          [](ShapeComputeGraphMapping& g) {
+            return g.graph_output_to_symbolic_shape_dim_;
+          });
 }
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
