@@ -1,41 +1,49 @@
+# mypy: allow-untyped-defs
 import functools
 import math
-import numbers
 import operator
 import weakref
-from typing import List
+from typing import Optional
 
 import torch
 import torch.nn.functional as F
 from torch.distributions import constraints
-from torch.distributions.utils import (_sum_rightmost, broadcast_all,
-                                       lazy_property, tril_matrix_to_vec,
-                                       vec_to_tril_matrix)
-from torch.nn.functional import pad
-from torch.nn.functional import softplus
+from torch.distributions.utils import (
+    _sum_rightmost,
+    broadcast_all,
+    lazy_property,
+    tril_matrix_to_vec,
+    vec_to_tril_matrix,
+)
+from torch.nn.functional import pad, softplus
+from torch.types import _Number
+
 
 __all__ = [
-    'AbsTransform',
-    'AffineTransform',
-    'CatTransform',
-    'ComposeTransform',
-    'CorrCholeskyTransform',
-    'ExpTransform',
-    'IndependentTransform',
-    'LowerCholeskyTransform',
-    'PowerTransform',
-    'ReshapeTransform',
-    'SigmoidTransform',
-    'TanhTransform',
-    'SoftmaxTransform',
-    'StackTransform',
-    'StickBreakingTransform',
-    'Transform',
-    'identity_transform',
+    "AbsTransform",
+    "AffineTransform",
+    "CatTransform",
+    "ComposeTransform",
+    "CorrCholeskyTransform",
+    "CumulativeDistributionTransform",
+    "ExpTransform",
+    "IndependentTransform",
+    "LowerCholeskyTransform",
+    "PositiveDefiniteTransform",
+    "PowerTransform",
+    "ReshapeTransform",
+    "SigmoidTransform",
+    "SoftplusTransform",
+    "TanhTransform",
+    "SoftmaxTransform",
+    "StackTransform",
+    "StickBreakingTransform",
+    "Transform",
+    "identity_transform",
 ]
 
 
-class Transform(object):
+class Transform:
     """
     Abstract class for invertable transformations with computable log
     det jacobians. They are primarily used in
@@ -79,29 +87,35 @@ class Transform(object):
             should be +1 or -1 depending on whether transform is monotone
             increasing or decreasing.
     """
+
     bijective = False
     domain: constraints.Constraint
     codomain: constraints.Constraint
 
     def __init__(self, cache_size=0):
         self._cache_size = cache_size
-        self._inv = None
+        self._inv: Optional[weakref.ReferenceType[Transform]] = None
         if cache_size == 0:
             pass  # default behavior
         elif cache_size == 1:
             self._cached_x_y = None, None
         else:
-            raise ValueError('cache_size must be 0 or 1')
-        super(Transform, self).__init__()
+            raise ValueError("cache_size must be 0 or 1")
+        super().__init__()
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state["_inv"] = None
+        return state
 
     @property
-    def event_dim(self):
+    def event_dim(self) -> int:
         if self.domain.event_dim == self.codomain.event_dim:
             return self.domain.event_dim
         raise ValueError("Please use either .domain.event_dim or .codomain.event_dim")
 
     @property
-    def inv(self):
+    def inv(self) -> "Transform":
         """
         Returns the inverse :class:`Transform` of this transform.
         This should satisfy ``t.inv.inv is t``.
@@ -115,7 +129,7 @@ class Transform(object):
         return inv
 
     @property
-    def sign(self):
+    def sign(self) -> int:
         """
         Returns the sign of the determinant of the Jacobian, if applicable.
         In general this only makes sense for bijective transforms.
@@ -127,7 +141,7 @@ class Transform(object):
             return self
         if type(self).__init__ is Transform.__init__:
             return type(self)(cache_size=cache_size)
-        raise NotImplementedError("{}.with_cache is not implemented".format(type(self)))
+        raise NotImplementedError(f"{type(self)}.with_cache is not implemented")
 
     def __eq__(self, other):
         return self is other
@@ -181,7 +195,7 @@ class Transform(object):
         raise NotImplementedError
 
     def __repr__(self):
-        return self.__class__.__name__ + '()'
+        return self.__class__.__name__ + "()"
 
     def forward_shape(self, shape):
         """
@@ -203,9 +217,10 @@ class _InverseTransform(Transform):
     Inverts a single :class:`Transform`.
     This class is private; please instead use the ``Transform.inv`` property.
     """
+
     def __init__(self, transform: Transform):
-        super(_InverseTransform, self).__init__(cache_size=transform._cache_size)
-        self._inv: Transform = transform
+        super().__init__(cache_size=transform._cache_size)
+        self._inv: Transform = transform  # type: ignore[assignment]
 
     @constraints.dependent_property(is_discrete=False)
     def domain(self):
@@ -218,17 +233,17 @@ class _InverseTransform(Transform):
         return self._inv.domain
 
     @property
-    def bijective(self):
+    def bijective(self) -> bool:  # type: ignore[override]
         assert self._inv is not None
         return self._inv.bijective
 
     @property
-    def sign(self):
+    def sign(self) -> int:
         assert self._inv is not None
         return self._inv.sign
 
     @property
-    def inv(self):
+    def inv(self) -> Transform:
         return self._inv
 
     def with_cache(self, cache_size=1):
@@ -269,10 +284,11 @@ class ComposeTransform(Transform):
         cache_size (int): Size of cache. If zero, no caching is done. If one,
             the latest single value is cached. Only 0 and 1 are supported.
     """
-    def __init__(self, parts: List[Transform], cache_size=0):
+
+    def __init__(self, parts: list[Transform], cache_size=0):
         if cache_size:
             parts = [part.with_cache(cache_size) for part in parts]
-        super(ComposeTransform, self).__init__(cache_size=cache_size)
+        super().__init__(cache_size=cache_size)
         self.parts = parts
 
     def __eq__(self, other):
@@ -311,18 +327,18 @@ class ComposeTransform(Transform):
         return codomain
 
     @lazy_property
-    def bijective(self):
+    def bijective(self) -> bool:  # type: ignore[override]
         return all(p.bijective for p in self.parts)
 
     @lazy_property
-    def sign(self):
+    def sign(self) -> int:  # type: ignore[override]
         sign = 1
         for p in self.parts:
             sign = sign * p.sign
         return sign
 
     @property
-    def inv(self):
+    def inv(self) -> Transform:
         inv = None
         if self._inv is not None:
             inv = self._inv()
@@ -355,8 +371,11 @@ class ComposeTransform(Transform):
         terms = []
         event_dim = self.domain.event_dim
         for part, x, y in zip(self.parts, xs[:-1], xs[1:]):
-            terms.append(_sum_rightmost(part.log_abs_det_jacobian(x, y),
-                                        event_dim - part.domain.event_dim))
+            terms.append(
+                _sum_rightmost(
+                    part.log_abs_det_jacobian(x, y), event_dim - part.domain.event_dim
+                )
+            )
             event_dim += part.codomain.event_dim - part.domain.event_dim
         return functools.reduce(operator.add, terms)
 
@@ -371,9 +390,9 @@ class ComposeTransform(Transform):
         return shape
 
     def __repr__(self):
-        fmt_string = self.__class__.__name__ + '(\n    '
-        fmt_string += ',\n    '.join([p.__repr__() for p in self.parts])
-        fmt_string += '\n)'
+        fmt_string = self.__class__.__name__ + "(\n    "
+        fmt_string += ",\n    ".join([p.__repr__() for p in self.parts])
+        fmt_string += "\n)"
         return fmt_string
 
 
@@ -393,6 +412,7 @@ class IndependentTransform(Transform):
         reinterpreted_batch_ndims (int): The number of extra rightmost
             dimensions to treat as dependent.
     """
+
     def __init__(self, base_transform, reinterpreted_batch_ndims, cache_size=0):
         super().__init__(cache_size=cache_size)
         self.base_transform = base_transform.with_cache(cache_size)
@@ -401,26 +421,28 @@ class IndependentTransform(Transform):
     def with_cache(self, cache_size=1):
         if self._cache_size == cache_size:
             return self
-        return IndependentTransform(self.base_transform,
-                                    self.reinterpreted_batch_ndims,
-                                    cache_size=cache_size)
+        return IndependentTransform(
+            self.base_transform, self.reinterpreted_batch_ndims, cache_size=cache_size
+        )
 
     @constraints.dependent_property(is_discrete=False)
     def domain(self):
-        return constraints.independent(self.base_transform.domain,
-                                       self.reinterpreted_batch_ndims)
+        return constraints.independent(
+            self.base_transform.domain, self.reinterpreted_batch_ndims
+        )
 
     @constraints.dependent_property(is_discrete=False)
     def codomain(self):
-        return constraints.independent(self.base_transform.codomain,
-                                       self.reinterpreted_batch_ndims)
+        return constraints.independent(
+            self.base_transform.codomain, self.reinterpreted_batch_ndims
+        )
 
     @property
-    def bijective(self):
+    def bijective(self) -> bool:  # type: ignore[override]
         return self.base_transform.bijective
 
     @property
-    def sign(self):
+    def sign(self) -> int:  # type: ignore[override]
         return self.base_transform.sign
 
     def _call(self, x):
@@ -458,7 +480,10 @@ class ReshapeTransform(Transform):
     Arguments:
         in_shape (torch.Size): The input event shape.
         out_shape (torch.Size): The output event shape.
+        cache_size (int): Size of cache. If zero, no caching is done. If one,
+            the latest single value is cached. Only 0 and 1 are supported. (Default 0.)
     """
+
     bijective = True
 
     def __init__(self, in_shape, out_shape, cache_size=0):
@@ -482,15 +507,15 @@ class ReshapeTransform(Transform):
         return ReshapeTransform(self.in_shape, self.out_shape, cache_size=cache_size)
 
     def _call(self, x):
-        batch_shape = x.shape[:x.dim() - len(self.in_shape)]
+        batch_shape = x.shape[: x.dim() - len(self.in_shape)]
         return x.reshape(batch_shape + self.out_shape)
 
     def _inverse(self, y):
-        batch_shape = y.shape[:y.dim() - len(self.out_shape)]
+        batch_shape = y.shape[: y.dim() - len(self.out_shape)]
         return y.reshape(batch_shape + self.in_shape)
 
     def log_abs_det_jacobian(self, x, y):
-        batch_shape = x.shape[:x.dim() - len(self.in_shape)]
+        batch_shape = x.shape[: x.dim() - len(self.in_shape)]
         return x.new_zeros(batch_shape)
 
     def forward_shape(self, shape):
@@ -498,7 +523,9 @@ class ReshapeTransform(Transform):
             raise ValueError("Too few dimensions on input")
         cut = len(shape) - len(self.in_shape)
         if shape[cut:] != self.in_shape:
-            raise ValueError("Shape mismatch: expected {} but got {}".format(shape[cut:], self.in_shape))
+            raise ValueError(
+                f"Shape mismatch: expected {shape[cut:]} but got {self.in_shape}"
+            )
         return shape[:cut] + self.out_shape
 
     def inverse_shape(self, shape):
@@ -506,7 +533,9 @@ class ReshapeTransform(Transform):
             raise ValueError("Too few dimensions on input")
         cut = len(shape) - len(self.out_shape)
         if shape[cut:] != self.out_shape:
-            raise ValueError("Shape mismatch: expected {} but got {}".format(shape[cut:], self.out_shape))
+            raise ValueError(
+                f"Shape mismatch: expected {shape[cut:]} but got {self.out_shape}"
+            )
         return shape[:cut] + self.in_shape
 
 
@@ -514,6 +543,7 @@ class ExpTransform(Transform):
     r"""
     Transform via the mapping :math:`y = \exp(x)`.
     """
+
     domain = constraints.real
     codomain = constraints.positive
     bijective = True
@@ -536,20 +566,24 @@ class PowerTransform(Transform):
     r"""
     Transform via the mapping :math:`y = x^{\text{exponent}}`.
     """
+
     domain = constraints.positive
     codomain = constraints.positive
     bijective = True
-    sign = +1
 
     def __init__(self, exponent, cache_size=0):
-        super(PowerTransform, self).__init__(cache_size=cache_size)
-        self.exponent, = broadcast_all(exponent)
+        super().__init__(cache_size=cache_size)
+        (self.exponent,) = broadcast_all(exponent)
 
     def with_cache(self, cache_size=1):
         if self._cache_size == cache_size:
             return self
         return PowerTransform(self.exponent, cache_size=cache_size)
 
+    @lazy_property
+    def sign(self) -> int:  # type: ignore[override]
+        return self.exponent.sign()
+
     def __eq__(self, other):
         if not isinstance(other, PowerTransform):
             return False
@@ -573,13 +607,14 @@ class PowerTransform(Transform):
 
 def _clipped_sigmoid(x):
     finfo = torch.finfo(x.dtype)
-    return torch.clamp(torch.sigmoid(x), min=finfo.tiny, max=1. - finfo.eps)
+    return torch.clamp(torch.sigmoid(x), min=finfo.tiny, max=1.0 - finfo.eps)
 
 
 class SigmoidTransform(Transform):
     r"""
     Transform via the mapping :math:`y = \frac{1}{1 + \exp(-x)}` and :math:`x = \text{logit}(y)`.
     """
+
     domain = constraints.real
     codomain = constraints.unit_interval
     bijective = True
@@ -593,27 +628,60 @@ class SigmoidTransform(Transform):
 
     def _inverse(self, y):
         finfo = torch.finfo(y.dtype)
-        y = y.clamp(min=finfo.tiny, max=1. - finfo.eps)
+        y = y.clamp(min=finfo.tiny, max=1.0 - finfo.eps)
         return y.log() - (-y).log1p()
 
     def log_abs_det_jacobian(self, x, y):
         return -F.softplus(-x) - F.softplus(x)
 
 
+class SoftplusTransform(Transform):
+    r"""
+    Transform via the mapping :math:`\text{Softplus}(x) = \log(1 + \exp(x))`.
+    The implementation reverts to the linear function when :math:`x > 20`.
+    """
+
+    domain = constraints.real
+    codomain = constraints.positive
+    bijective = True
+    sign = +1
+
+    def __eq__(self, other):
+        return isinstance(other, SoftplusTransform)
+
+    def _call(self, x):
+        return softplus(x)
+
+    def _inverse(self, y):
+        return (-y).expm1().neg().log() + y
+
+    def log_abs_det_jacobian(self, x, y):
+        return -softplus(-x)
+
+
 class TanhTransform(Transform):
     r"""
     Transform via the mapping :math:`y = \tanh(x)`.
 
     It is equivalent to
-    ```
-    ComposeTransform([AffineTransform(0., 2.), SigmoidTransform(), AffineTransform(-1., 2.)])
-    ```
+
+    .. code-block:: python
+
+        ComposeTransform(
+            [
+                AffineTransform(0.0, 2.0),
+                SigmoidTransform(),
+                AffineTransform(-1.0, 2.0),
+            ]
+        )
+
     However this might not be numerically stable, thus it is recommended to use `TanhTransform`
     instead.
 
     Note that one should use `cache_size=1` when it comes to `NaN/Inf` values.
 
     """
+
     domain = constraints.real
     codomain = constraints.interval(-1.0, 1.0)
     bijective = True
@@ -633,13 +701,12 @@ class TanhTransform(Transform):
     def log_abs_det_jacobian(self, x, y):
         # We use a formula that is more numerically stable, see details in the following link
         # https://github.com/tensorflow/probability/blob/master/tensorflow_probability/python/bijectors/tanh.py#L69-L80
-        return 2. * (math.log(2.) - x - softplus(-2. * x))
+        return 2.0 * (math.log(2.0) - x - softplus(-2.0 * x))
 
 
 class AbsTransform(Transform):
-    r"""
-    Transform via the mapping :math:`y = |x|`.
-    """
+    r"""Transform via the mapping :math:`y = |x|`."""
+
     domain = constraints.real
     codomain = constraints.positive
 
@@ -664,16 +731,17 @@ class AffineTransform(Transform):
             for univariate random variables, 1 for distributions over vectors,
             2 for distributions over matrices, etc.
     """
+
     bijective = True
 
     def __init__(self, loc, scale, event_dim=0, cache_size=0):
-        super(AffineTransform, self).__init__(cache_size=cache_size)
+        super().__init__(cache_size=cache_size)
         self.loc = loc
         self.scale = scale
         self._event_dim = event_dim
 
     @property
-    def event_dim(self):
+    def event_dim(self) -> int:
         return self._event_dim
 
     @constraints.dependent_property(is_discrete=False)
@@ -691,20 +759,22 @@ class AffineTransform(Transform):
     def with_cache(self, cache_size=1):
         if self._cache_size == cache_size:
             return self
-        return AffineTransform(self.loc, self.scale, self.event_dim, cache_size=cache_size)
+        return AffineTransform(
+            self.loc, self.scale, self.event_dim, cache_size=cache_size
+        )
 
     def __eq__(self, other):
         if not isinstance(other, AffineTransform):
             return False
 
-        if isinstance(self.loc, numbers.Number) and isinstance(other.loc, numbers.Number):
+        if isinstance(self.loc, _Number) and isinstance(other.loc, _Number):
             if self.loc != other.loc:
                 return False
         else:
             if not (self.loc == other.loc).all().item():
                 return False
 
-        if isinstance(self.scale, numbers.Number) and isinstance(other.scale, numbers.Number):
+        if isinstance(self.scale, _Number) and isinstance(other.scale, _Number):
             if self.scale != other.scale:
                 return False
         else:
@@ -714,8 +784,8 @@ class AffineTransform(Transform):
         return True
 
     @property
-    def sign(self):
-        if isinstance(self.scale, numbers.Real):
+    def sign(self) -> int:
+        if isinstance(self.scale, _Number):
             return 1 if float(self.scale) > 0 else -1 if float(self.scale) < 0 else 0
         return self.scale.sign()
 
@@ -728,25 +798,25 @@ class AffineTransform(Transform):
     def log_abs_det_jacobian(self, x, y):
         shape = x.shape
         scale = self.scale
-        if isinstance(scale, numbers.Real):
+        if isinstance(scale, _Number):
             result = torch.full_like(x, math.log(abs(scale)))
         else:
             result = torch.abs(scale).log()
         if self.event_dim:
-            result_size = result.size()[:-self.event_dim] + (-1,)
+            result_size = result.size()[: -self.event_dim] + (-1,)
             result = result.view(result_size).sum(-1)
-            shape = shape[:-self.event_dim]
+            shape = shape[: -self.event_dim]
         return result.expand(shape)
 
     def forward_shape(self, shape):
-        return torch.broadcast_shapes(shape,
-                                      getattr(self.loc, "shape", ()),
-                                      getattr(self.scale, "shape", ()))
+        return torch.broadcast_shapes(
+            shape, getattr(self.loc, "shape", ()), getattr(self.scale, "shape", ())
+        )
 
     def inverse_shape(self, shape):
-        return torch.broadcast_shapes(shape,
-                                      getattr(self.loc, "shape", ()),
-                                      getattr(self.scale, "shape", ()))
+        return torch.broadcast_shapes(
+            shape, getattr(self.loc, "shape", ()), getattr(self.scale, "shape", ())
+        )
 
 
 class CorrCholeskyTransform(Transform):
@@ -765,6 +835,7 @@ class CorrCholeskyTransform(Transform):
            - Applies :math:`s_i = StickBreakingTransform(z_i)`.
            - Transforms back into signed domain: :math:`y_i = sign(r_i) * \sqrt{s_i}`.
     """
+
     domain = constraints.real_vector
     codomain = constraints.corr_cholesky
     bijective = True
@@ -777,7 +848,7 @@ class CorrCholeskyTransform(Transform):
         # apply stick-breaking on the squared values
         # Note that y = sign(r) * sqrt(z * z1m_cumprod)
         #             = (sign(r) * sqrt(z)) * sqrt(z1m_cumprod) = r * sqrt(z1m_cumprod)
-        z = r ** 2
+        z = r**2
         z1m_cumprod_sqrt = (1 - z).sqrt().cumprod(-1)
         # Diagonal elements must be 1.
         r = r + torch.eye(r.shape[-1], dtype=r.dtype, device=r.device)
@@ -793,7 +864,7 @@ class CorrCholeskyTransform(Transform):
         y_cumsum_vec = tril_matrix_to_vec(y_cumsum_shifted, diag=-1)
         t = y_vec / (y_cumsum_vec).sqrt()
         # inverse of tanh
-        x = ((1 + t) / (1 - t)).log() / 2
+        x = (t.log1p() - t.neg().log1p()) / 2
         return x
 
     def log_abs_det_jacobian(self, x, y, intermediates=None):
@@ -807,7 +878,7 @@ class CorrCholeskyTransform(Transform):
         # also works for 2 x 2 matrix
         y1m_cumsum_tril = tril_matrix_to_vec(y1m_cumsum, diag=-2)
         stick_breaking_logdet = 0.5 * (y1m_cumsum_tril).log().sum(-1)
-        tanh_logdet = -2 * (x + softplus(-2 * x) - math.log(2.)).sum(dim=-1)
+        tanh_logdet = -2 * (x + softplus(-2 * x) - math.log(2.0)).sum(dim=-1)
         return stick_breaking_logdet + tanh_logdet
 
     def forward_shape(self, shape):
@@ -840,6 +911,7 @@ class SoftmaxTransform(Transform):
     coordinate-wise (except for the final normalization), and thus is
     appropriate for coordinate-wise optimization algorithms.
     """
+
     domain = constraints.real_vector
     codomain = constraints.simplex
 
@@ -879,6 +951,7 @@ class StickBreakingTransform(Transform):
     This is bijective and appropriate for use in HMC; however it mixes
     coordinates together and is less appropriate for optimization.
     """
+
     domain = constraints.real_vector
     codomain = constraints.simplex
     bijective = True
@@ -929,6 +1002,7 @@ class LowerCholeskyTransform(Transform):
     This is useful for parameterizing positive definite matrices in terms of
     their Cholesky factorization.
     """
+
     domain = constraints.independent(constraints.real, 2)
     codomain = constraints.lower_cholesky
 
@@ -942,25 +1016,48 @@ class LowerCholeskyTransform(Transform):
         return y.tril(-1) + y.diagonal(dim1=-2, dim2=-1).log().diag_embed()
 
 
+class PositiveDefiniteTransform(Transform):
+    """
+    Transform from unconstrained matrices to positive-definite matrices.
+    """
+
+    domain = constraints.independent(constraints.real, 2)
+    codomain = constraints.positive_definite  # type: ignore[assignment]
+
+    def __eq__(self, other):
+        return isinstance(other, PositiveDefiniteTransform)
+
+    def _call(self, x):
+        x = LowerCholeskyTransform()(x)
+        return x @ x.mT
+
+    def _inverse(self, y):
+        y = torch.linalg.cholesky(y)
+        return LowerCholeskyTransform().inv(y)
+
+
 class CatTransform(Transform):
-    tseq: List[numbers.Number]
     """
     Transform functor that applies a sequence of transforms `tseq`
     component-wise to each submatrix at `dim`, of length `lengths[dim]`,
     in a way compatible with :func:`torch.cat`.
 
     Example::
+
        x0 = torch.cat([torch.range(1, 10), torch.range(1, 10)], dim=0)
        x = torch.cat([x0, x0], dim=0)
        t0 = CatTransform([ExpTransform(), identity_transform], dim=0, lengths=[10, 10])
        t = CatTransform([t0, t0], dim=0, lengths=[20, 20])
        y = t(x)
     """
+
+    transforms: list[Transform]
+
     def __init__(self, tseq, dim=0, lengths=None, cache_size=0):
         assert all(isinstance(t, Transform) for t in tseq)
         if cache_size:
             tseq = [t.with_cache(cache_size) for t in tseq]
-        super(CatTransform, self).__init__(cache_size=cache_size)
+        super().__init__(cache_size=cache_size)
         self.transforms = list(tseq)
         if lengths is None:
             lengths = [1] * len(self.transforms)
@@ -969,17 +1066,17 @@ class CatTransform(Transform):
         self.dim = dim
 
     @lazy_property
-    def event_dim(self):
+    def event_dim(self) -> int:  # type: ignore[override]
         return max(t.event_dim for t in self.transforms)
 
     @lazy_property
-    def length(self):
+    def length(self) -> int:
         return sum(self.lengths)
 
     def with_cache(self, cache_size=1):
         if self._cache_size == cache_size:
             return self
-        return CatTransform(self.tseq, self.dim, self.lengths, cache_size)
+        return CatTransform(self.transforms, self.dim, self.lengths, cache_size)
 
     def _call(self, x):
         assert -x.dim() <= self.dim < x.dim()
@@ -1029,18 +1126,20 @@ class CatTransform(Transform):
             return sum(logdetjacs)
 
     @property
-    def bijective(self):
+    def bijective(self) -> bool:  # type: ignore[override]
         return all(t.bijective for t in self.transforms)
 
     @constraints.dependent_property
     def domain(self):
-        return constraints.cat([t.domain for t in self.transforms],
-                               self.dim, self.lengths)
+        return constraints.cat(
+            [t.domain for t in self.transforms], self.dim, self.lengths
+        )
 
     @constraints.dependent_property
     def codomain(self):
-        return constraints.cat([t.codomain for t in self.transforms],
-                               self.dim, self.lengths)
+        return constraints.cat(
+            [t.codomain for t in self.transforms], self.dim, self.lengths
+        )
 
 
 class StackTransform(Transform):
@@ -1050,15 +1149,19 @@ class StackTransform(Transform):
     in a way compatible with :func:`torch.stack`.
 
     Example::
+
        x = torch.stack([torch.range(1, 10), torch.range(1, 10)], dim=1)
        t = StackTransform([ExpTransform(), identity_transform], dim=1)
        y = t(x)
     """
+
+    transforms: list[Transform]
+
     def __init__(self, tseq, dim=0, cache_size=0):
         assert all(isinstance(t, Transform) for t in tseq)
         if cache_size:
             tseq = [t.with_cache(cache_size) for t in tseq]
-        super(StackTransform, self).__init__(cache_size=cache_size)
+        super().__init__(cache_size=cache_size)
         self.transforms = list(tseq)
         self.dim = dim
 
@@ -1099,7 +1202,7 @@ class StackTransform(Transform):
         return torch.stack(logdetjacs, dim=self.dim)
 
     @property
-    def bijective(self):
+    def bijective(self) -> bool:  # type: ignore[override]
         return all(t.bijective for t in self.transforms)
 
     @constraints.dependent_property
@@ -1109,3 +1212,49 @@ class StackTransform(Transform):
     @constraints.dependent_property
     def codomain(self):
         return constraints.stack([t.codomain for t in self.transforms], self.dim)
+
+
+class CumulativeDistributionTransform(Transform):
+    """
+    Transform via the cumulative distribution function of a probability distribution.
+
+    Args:
+        distribution (Distribution): Distribution whose cumulative distribution function to use for
+            the transformation.
+
+    Example::
+
+        # Construct a Gaussian copula from a multivariate normal.
+        base_dist = MultivariateNormal(
+            loc=torch.zeros(2),
+            scale_tril=LKJCholesky(2).sample(),
+        )
+        transform = CumulativeDistributionTransform(Normal(0, 1))
+        copula = TransformedDistribution(base_dist, [transform])
+    """
+
+    bijective = True
+    codomain = constraints.unit_interval
+    sign = +1
+
+    def __init__(self, distribution, cache_size=0):
+        super().__init__(cache_size=cache_size)
+        self.distribution = distribution
+
+    @property
+    def domain(self) -> constraints.Constraint:  # type: ignore[override]
+        return self.distribution.support
+
+    def _call(self, x):
+        return self.distribution.cdf(x)
+
+    def _inverse(self, y):
+        return self.distribution.icdf(y)
+
+    def log_abs_det_jacobian(self, x, y):
+        return self.distribution.log_prob(x)
+
+    def with_cache(self, cache_size=1):
+        if self._cache_size == cache_size:
+            return self
+        return CumulativeDistributionTransform(self.distribution, cache_size=cache_size)
